/*********************************************************************
 * Neophytos Iacovou, University of Minnesota DCS.
 *
 * Copyright 1991, 1992, 1993, 1994 
 * by the Regents of the University of Minnesota
 *
 * see the file "Copyright" in the distribution for conditions of use.
 *********************************************************************/

/*
 * Copyright (c) 1992 The Regents of the University of California.
 * All rights reserved.
 *
 * Author:  Ray Larson, ray@sherlock.berkeley.edu
 *      School of Library and Information Studies, UC Berkeley
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *  This product includes software developed by the University of
 *  California, Berkeley.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHOR ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "dispmarc.h" 




/*************************************************************************/
/* DispMARC - display a marc record given files and record ID            */
/*            requires an associator address file for the MARC file      */
/*            Parameters:                                                */
/*          marcin - MARC input file                                     */
/*          assocfile - Associator file for marcin                       */
/*          recordID - MARC record # to retrieve                         */
/*          format - one of the formats defined above                    */
/*          displaynum - number to use in display (if                    */
/*              -1 use the recordID.                                     */
/*          maxlen - maximum length of line to output                    */
/*          output - pointer to the output routine                       */
/*************************************************************************/
MARC_REC *DispMARC( marcin, assocfile, recordID, format, displaynum, maxlen, 
                    output, host, dbname, port, operator, mrecs, aCh_title, 
                    aCh_author, aCh_subject, aCh_marc_file, aCh_gopher_host, 
                    aCh_binary_path, N_gopher_port, B_info )

int marcin, assocfile; /*files*/
long recordID, format, displaynum, maxlen ;
void *(output)();
P_CHAR host, dbname, operator, mrecs;  
NUMBER port, N_gopher_port;
P_CHAR aCh_title, aCh_author, aCh_subject, aCh_marc_file, aCh_gopher_host;
P_CHAR aCh_binary_path; 
BOOL    B_info;

{
    MARC_DIRENTRY_OVER *dir;
    MARC_FIELD *fld;
    MARC_SUBFIELD *subf;
    MARC_REC *rec;
    long lrecl;
    char *readptr, *c;
    int  i;
    int  print_leader = FALSE;         /* special function flags */
    int  print_directory = FALSE;
    int  print_authority = FALSE;
    int  repeat = FALSE;
    char *line, *format_field();
    DISP_FORMAT *formatcontrol, *f;
    BOOL    B_once = TRUE;




    /*
     * Assign a display format
     */
    switch (format) 
    {
        case 1: formatcontrol = &shortformat[0];
                break;
        case 2: formatcontrol = &marcformat[0];
                break;
        case 3: formatcontrol = &evaluationformat[0];
                break;
        default: formatcontrol = &defaultformat[0];
                break;
    }

    /* find the address of the record and position the marc file pointer */
    if (assocfile && recordID > 0) {
        if (SeekMARC(marcin, assocfile, recordID) == -1)
            return(NULL);
    }

    /* Read the record from the marcin file */
    if ((lrecl = ReadMARC(marcin,recbuffer,sizeof recbuffer)) == 0)
        return(NULL);
    else   /* record was read - so load it into the marc structure */
    {
        if((rec = GetMARC(recbuffer,lrecl,0)) == NULL) {
            fprintf(stderr,"no record retrieved by GetMARC\n");
            return(NULL);
        }

        if (displaynum == -1) 
            displaynum = recordID;

        for (f = formatcontrol; f->label; f++) 
        {
    /* get the first field in the format */
            fld = GetField(rec, NULL, fieldbuffer, f->tags);

    /* if no field found, check for number format */
            if (fld == NULL && *f->tags == '\0') 
            {

    /* a null tag means output the supplied record number */
                sprintf(linebuffer,"%s%s%d%s", 
                    f->label, f->beginpunct,displaynum, f->endpunct);
        
    /* assume it won't be longer than maxlen */
                    outputline ( output,line, maxlen, f->indent, format, 
                                 B_once, host, dbname, port, operator, 
                                 mrecs, aCh_title, aCh_author, aCh_subject,
                                 aCh_marc_file, aCh_gopher_host, 
                                 aCh_binary_path, N_gopher_port, recordID,
                                 B_info );

            }   
            repeat = FALSE;
            
            while (fld) 
            {
                if (f->print_all) 
                {
                    codeconvert(fieldbuffer);

                    if (*f->label == '\0')
                    {
                        sprintf(linebuffer,"%s %s%s%s", 
                            fld->tag, f->beginpunct,fieldbuffer, f->endpunct);
                    }
                    else
                    {
                        sprintf(linebuffer,"%s %s%s%s", 
                            f->label, f->beginpunct,fieldbuffer, f->endpunct);
                    }

                    outputline ( output,line, maxlen, f->indent, format, 
                                 B_once, host, dbname, port, operator, 
                                 mrecs, aCh_title, aCh_author, aCh_subject,
                                 aCh_marc_file, aCh_gopher_host, 
                                 aCh_binary_path, N_gopher_port, recordID,
                                 B_info );

                }
                else  /* more selective printing */
                {
                    line = format_field( fld, f, linebuffer, repeat );

                    if ( 1 == format )
                    {
                        if ( TRUE == B_once)
                        {
                            if (line)
                            {
                                outputline ( output,line, maxlen, f->indent, 
                                        format, B_once, host, dbname, port, 
                                        operator, mrecs, aCh_title, aCh_author, 
                                         aCh_subject, aCh_marc_file, 
                                         aCh_gopher_host, aCh_binary_path, 
                                         N_gopher_port, recordID, B_info );

                                B_once = FALSE;
                            }
                        }
                        else
                        {
                           if (line)
                           {
                                outputline ( output,line, maxlen, f->indent, 
                                        format, B_once, host, dbname, port, 
                                        operator, mrecs, aCh_title, aCh_author,
                                         aCh_subject, aCh_marc_file,
                                         aCh_gopher_host, aCh_binary_path,
                                         N_gopher_port, recordID, B_info );
                           }
                        }
                    }
                    else
                    {
                        if (line) 
                        {
                            outputline ( output,line, maxlen, f->indent, format,
                                         B_once, host, dbname, port, operator, 
                                         mrecs, aCh_title, aCh_author, 
                                         aCh_subject, aCh_marc_file, 
                                         aCh_gopher_host, aCh_binary_path, 
                                         N_gopher_port, recordID, B_info );
                        }
                    }
                }

                /* more of the same tag set? */
                fld = GetField( NULL, fld->next, fieldbuffer, f->tags );
                if (fld)
                {
                    repeat = TRUE;
                }
            }
        }

        return(rec);
    }
}





/***********************************************************************/
/* format_field - given a marc field struct and a format item, build   */
/*                a line in a buffer according to the format.          */
/***********************************************************************/
char *format_field(mf,format,buff,repeat)
MARC_FIELD *mf;
DISP_FORMAT *format;
char *buff;
int repeat;
{
    MARC_SUBFIELD *subf;
    register char *linend, *c;
    char *index();
    int pos, count, ok;
    
    linend = buff;
    *linend = '\0';
    pos = 0;

    if (repeat && (format->repeatlabel == FALSE)) ;  /* skip it */
    else /* add the label */
        for(c = format->label; *c ; *linend++ = *c++) pos++;

    /* indentation */
    for (; pos < format->indent; pos++) *linend++ = ' ';

    /* initial 'punctuation' */     
    for(c = format->beginpunct; *c ; *linend++ = *c++);

    /* subfields */
    for (subf = mf->subfield; subf; subf = subf->next) {
        if ((*format->subfields == '\0') || 
            (index(format->subfields, subf->code))) {   

                /* this one should be copied */
                for(c = format->beginpunct; *c ; *linend++ = *c++);
        
                count = subfcopy(linend,subf->data,1) - 1;
                linend += count;

                for(c = format->subfsep; *c ; *linend++ = *c++);
                ok = TRUE;
        }
    }
    
    if (ok) {
        /* backtrack over the last subfield separator */
        linend -= strlen(format->subfsep);

        /* add end of field punctuation */
        if (*format->endpunct) {
            /* kill the existing punctuation and trailing blanks */
            if (ispunct(*(linend-1)) && ispunct(*format->endpunct) &&
               *(linend-1) != ')' && *(linend-1) != ']')
                linend--;

            while(*(linend-1) == ' ') linend--;
            for(c = format->endpunct; *c ; *linend++ = *c++) pos++;
        }

        *linend = '\0'; /* Null terminate the line */
        return(buff);   
    }
    else return(NULL); /* no subfields copied */
}





    /* 
     * break an output line into segments to fit on the screen and call the
     * output function
     */
outputline( outfunc, line, maxlen, indent, F_format, B_once, host, 
            dbname, port, operator, mrecs, aCh_title, aCh_author, 
            aCh_subject, aCh_marc_file, aCh_gopher_host, 
            aCh_binary_path, N_gopher_port, recordID, B_info )

    void    *(outfunc)();
    char    *line;
    int     maxlen, indent;
    FLAG    F_format;
    P_CHAR  host, dbname, operator, mrecs;
    NUMBER  port, N_gopher_port;
    P_CHAR  aCh_title, aCh_author, aCh_subject;
    P_CHAR  aCh_marc_file, aCh_gopher_host, aCh_binary_path;
    BOOL    B_once, B_info;
    long    recordID;

{
    int     linelen, i;
    char    indentstr[80];
    char    *nextpart, *c;
    CHAR    rCh_line2 [FIELDBUFSIZE];

    /* the format strings */
    STRING  Sz_format1 = "0%s\texec:-h %s -c %s -p %i -o %s -m %i -T \"%s\" -A \"%s\" -S \"%s\" -M %s -B %s -H %s -P %i -f %li:%s\t%s\t%i\r\n";
    STRING  Sz_format2 = "i%s\t\t%s\t%i\r\n";



  /* if the line will fit output it now */
  if ((linelen = strlen(line)) <= maxlen) {

    if ( 1 == F_format )
    {

        if ( TRUE == B_once )
        {
            if ( B_info )
            {
                printf( "+INFO: " );
            }

            printf( Sz_format1, line, host, dbname, port, operator, mrecs, 
                    aCh_title, aCh_author, aCh_subject, aCh_marc_file,
                    aCh_binary_path, aCh_gopher_host, N_gopher_port, recordID,
                    aCh_binary_path, aCh_gopher_host, N_gopher_port );

            return;
        }
        else
        {
    /*
     *      if ( B_info )
     *      {
     *          printf( "+INFO: " );
     *      }
     *          printf( Sz_format2, line, aCh_gopher_host, N_gopher_port );
     */ 
            return;
        }
    }
    else
    {
        printf( line );
        return;
    }

  }
  else { /* put out first part, no indentation */
    for (c = &line[maxlen - 1]; *c != ' '; c--); /* find word break */
    *c = '\0';
    nextpart = c+1;

    if ( 1 == F_format )
    {

        if ( TRUE == B_once )
        {
            if ( B_info )
            {
                printf( "+INFO: " );
            }

            printf( Sz_format1, line, host, dbname, port, operator, mrecs, 
                    aCh_title, aCh_author, aCh_subject, aCh_marc_file,
                    aCh_binary_path, aCh_gopher_host, N_gopher_port, recordID,
                    aCh_binary_path, aCh_gopher_host, N_gopher_port );

            return;
        }
        else
        {
            if ( B_info )
            {
                printf( "+INFO: " );

                printf( Sz_format2, line, aCh_gopher_host, N_gopher_port );
            }

            return;
        }
    }
    else
    {
        printf(line);
        (*outfunc)("\n");
    }

  }
  /* set up indent string - add 3 spaces to regular indent for wrapped lines */
  indent += 3;

    for (i=0;i<indent; i++) indentstr[i] = ' ';
  indentstr[i] = '\0';

  /* loop to output rest of line */
  while ((linelen = strlen(nextpart)) > (maxlen - indent)) {
        for (c = &nextpart[maxlen - indent]; *c != ' '; c--); 
             /* find word break */ 
        *c = '\0'; 
        (*outfunc)(indentstr); 
        (*outfunc)(nextpart); 
        (*outfunc)("\r\n");
        nextpart = c+1;  
  }
  (*outfunc)(indentstr); 
  (*outfunc)(nextpart); 
  return;
}








    /*
     * given a filename of the MARC records to be processed it builds
     * an associator file
     */ 

VOID
OpenAssocFile( char *inputfilename, char *assocfilename )
{
  long lrecl, nrecs, recordID, recoffset;
  int  marcin, assoc, i;
  char recbuffer[LENGBUFFER];
 
  /* open the input file */

  marcin = open(inputfilename, READONLY);
  if (marcin == BADFILE) {
    printf("cannot open marc input file %s\n", inputfilename);
    exit(0);
  }
  /* open the 'associator' index file */
  /*sprintf(assocfilename,"%s.assoc",inputfilename);*/
  assoc = open(assocfilename,OUTPUT,RWFLAGS);
  if(assoc == BADFILE) {
    if (errno == EEXIST)
      printf("Index file %s already exists \n",assocfilename);
    else if (errno == ENOENT)
      printf("cannot open associator index file \n");
    else 
      printf("open error %d on associator index file \n", errno);

    exit(0);
  }

  /* skip through the input file, accumulating record lengths     */
  /* and write a 4 byte offsets to the associator file for each   */
  /* record read                                                  */

  /* write a null zero record for the associator */
  recordID = 0;
  recoffset = 0;
  write(assoc,&recoffset,sizeof(long));
  recordID++;

  while (read(marcin, recbuffer, 5) == 5) {
    /* verify that the buffer does contain digits */
    for(i=0; i<5; i++) {
      if (isdigit(recbuffer[i])) ; /* keep going */
      else {
    if (recoffset = rescan(marcin,recbuffer)) ;
    else {
      fprintf(stderr,"Bad record - unable to rescan #%d\n",recordID+1);
                    exit(1);
    }
      }
    }

    if((lrecl = GetNum(recbuffer, 5)) > 0)
      {
    write(assoc,&recoffset,sizeof(long));
    recordID++;
    recoffset = lseek(marcin, lrecl-5, SEEK_CUR);
      }
  }
  recordID--;
  recoffset -= lrecl;
  lseek(assoc, 0, 0);
  write(assoc,&recordID,sizeof(long));
  close(marcin);
  close(assoc);

    /*
     *printf("Number of records processed: %d  Last record offset %d\n", 
     *      recordID,recoffset);
     */

}

