/*********************************************************************
 * Neophytos Iacovou, University of Minnesota DCS.
 *
 * Copyright 1991, 1992, 1993, 1994 
 * by the Regents of the University of Minnesota
 *
 * see the file "Copyright" in the distribution for conditions of use.
 *********************************************************************/

/*
 * Copyright (c) 1992 The Regents of the University of California.
 * All rights reserved.
 *
 * Author:  Ray Larson, ray@sherlock.berkeley.edu
 *      School of Library and Information Studies, UC Berkeley
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *  This product includes software developed by the University of
 *  California, Berkeley.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHOR ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*-----------------------------------------------------------------------------
 * Header File: dispmarc.h 
 *
 *----------------------------------------------------------------------------*/
#ifndef _DISPMARC_H_
#define _DISPMARC_H_



#include <stdio.h>
#include <ctype.h>
#include <fcntl.h>
#include "marc.h"
#include "memcntl.h"
#include "standard.h"



    /* 
     * Defines and Macros
     */

#define MENUHT 3

#define   RECBUFSIZE     10000
#define   FIELDBUFSIZE   10000
#define   READONLY       O_RDONLY
#define   BADFILE        -1

#define   LENGBUFFER     10
#define   OUTPUT         O_CREAT+O_WRONLY+O_EXCL
#define   RWFLAGS        00400 | 00200 | 00040 | 00004
#define   RECTERM        '\035'


#ifndef   SEEK_CUR
#define   SEEK_CUR       1
#endif





    /*
     * Functions
     */

/* EXTERNAL ROUTINES -- in marclib.c and memcntl.c */
extern char *AllocSafe();
extern long GetNum(/*start char, number of chars to convert */);
extern long ReadMARC(/* file id, buffer, buffer size*/);
extern MARC_FIELD *SetField(/*rec, dir*/);
extern MARC_REC *GetMARC(/*buffer,record length, copy flag*/);
extern MARC_FIELD *GetField(/*marc_rec pointer,buffer,field tag*/);
extern MARC_SUBFIELD *GetSubf(/*marc_field pointer,buffer,subfield code*/);





    /*
     * Structures and Typedefs
     */

struct MemBlock *RememberKey;

/* EXTERNAL VARIABLES */
extern struct MemBlock *RememberKey;    /* key for memory allocation */
char recbuffer[RECBUFSIZE];
char fieldbuffer[FIELDBUFSIZE];
char linebuffer[FIELDBUFSIZE];

typedef struct {
    char *label;
    char *tags;
    char *subfields;
    char *beginpunct;
    char *subfsep;
    char *endpunct;
    int  newfield;
    int  print_all;
    int  print_indicators;
    int  print_delimiters;
    int  repeatlabel;
    int  indent;
 } DISP_FORMAT;


DISP_FORMAT defaultformat[] = {
/*
     {"Record #", "", "",""," ","\n",     TRUE,FALSE,FALSE,FALSE, FALSE, 0},
*/
     {"Author:" , "1xx", "", ""," ", ".\n",  TRUE,FALSE,FALSE,FALSE,FALSE, 15},
     {"Title:"  , "245", "", ""," ", ".\n",  TRUE,FALSE,FALSE,FALSE,FALSE, 15},
     {"Publisher:", "260", "", ""," ", ".\n",  TRUE,FALSE,FALSE,FALSE,FALSE, 15},
     {"Pages:"  , "300", "", ""," ", ".\n",  TRUE,FALSE,FALSE,FALSE,FALSE, 15},
     {"Series:" , "4xx", "", ""," ", "\n", TRUE,FALSE,FALSE,FALSE,FALSE, 15},
     {"Notes:"  , "5xx", "", "", " ","\n",   TRUE,FALSE,FALSE,FALSE,FALSE, 15},
     {"Subjects:","6xx", "", "", " -- ",".\n",  TRUE,FALSE,FALSE,FALSE,FALSE, 15},
     {"Other authors:","7xx", "", "", " ",".\n",  TRUE,FALSE,FALSE,FALSE,FALSE, 15},
     {"Call Numbers:","950", "", "", " ","\n",TRUE,FALSE,FALSE,FALSE,FALSE,15},
     {NULL,NULL,NULL,NULL,NULL,NULL,FALSE,FALSE,FALSE,FALSE,FALSE,0}
};


DISP_FORMAT shortformat[] =  {
/*
 *    {"Record #", "", "",""," ","\n",     TRUE,FALSE,FALSE,FALSE, FALSE, 0},
 *    {"Author:" , "1xx", "", ""," ", ".\n",  TRUE,FALSE,FALSE,FALSE,FALSE, 15},
 *    {"Title:"  , "245", "", ""," ", ".\n",  TRUE,FALSE,FALSE,FALSE,FALSE, 15},
 *    {NULL,NULL,NULL,NULL,NULL,NULL,FALSE,FALSE,FALSE,FALSE,FALSE,0}
 */

    { ""  , "245", "", ""," ", ".",  TRUE,FALSE,FALSE,FALSE,FALSE, 0 },
    { "" , "1xx", "", ""," ", ".",  TRUE,FALSE,FALSE,FALSE,FALSE, 0 },
    {NULL,NULL,NULL,NULL,NULL,NULL,FALSE,FALSE,FALSE,FALSE,FALSE,0}

};



 DISP_FORMAT marcformat[] =  {
/*
     {"Record ID: ", "", "",""," ","\n", TRUE,FALSE,FALSE,FALSE,FALSE, 0},
*/
     {"" , "xxx", "", "","", "\n",  TRUE,TRUE,TRUE,TRUE,FALSE,0},
     {NULL,NULL,NULL,NULL,NULL,NULL,FALSE,FALSE,FALSE,FALSE,FALSE,0}
};

DISP_FORMAT evaluationformat[] = {
/*
     {"Record #", "", "",""," ","\n",     TRUE,FALSE,FALSE,FALSE, FALSE, 0},
*/
     {" ", "", "",""," "," ",     TRUE,FALSE,FALSE,FALSE, FALSE, 0},
     {" "  , "245", "", ""," ", ".\n",  TRUE,FALSE,FALSE,FALSE,FALSE, 15},
/*
     {"Title:"  , "245", "", ""," ", ".",  TRUE,FALSE,FALSE,FALSE,FALSE, 15},
     {"Subjects:","6xx", "", "", " -- ",".\n",  TRUE,FALSE,FALSE,FALSE,FALSE, 15},
     {"Call Numbers:","950", "", "", " ","\n",TRUE,FALSE,FALSE,FALSE,FALSE,15},
*/
     {NULL,NULL,NULL,NULL,NULL,NULL,FALSE,FALSE,FALSE,FALSE,FALSE,0}
};



    /*
     * Variables
     */


#endif /* _DISPMARC_H_ */
