/***************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery
and Retrieval, 1994.

Permission to use, copy, modify, distribute, and sell this software
and its documentation, in whole or in part, for any purpose is hereby
granted without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included
in this distribution must remain intact.
2. Users of this software agree to make their best efforts (a) to
return to MCNC any improvements or extensions that they make, so that
these may be included in future releases; and (b) to inform MCNC/CNIDR
of noteworthy uses of this software.

3. The names of MCNC and Clearinghouse for Networked Information
Discovery and Retrieval may not be used in any advertising or publicity
relating to the software without the specific, prior written permission
of MCNC/CNIDR.

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

***************************************************************/
#include "platform.h"
#ifdef PLATFORM_MSDOS
#define MAXPROFILESTR 256
#ifdef PLATFORM_BORLANDC
#include <alloc.h>
#endif
#ifdef PLATFORM_MSC
#include <malloc.h>
#endif
#endif
#include <string.h>
#include <stdio.h>
#include "netbox.h"
#include "strbox.h"
#include "berutil.h"
#include "zdefs.h"
#include "init.h"
#include "search.h"
#include "present.h"
#include "zdecode.h"
#include "zencode.h"
#include "query.h"
#include "log.h"
#include "rpn.h"
#include "zutils.h"
#include "freemem.h"
#include "ztags.h"
#include "access.h"

#define NEW(a) (a*)calloc(1,sizeof(a))
/*
FUNCTION:	fwAccessControlRequest()
PURPOSE:	Sends an AccessControlRequest over the wire.
PRE:		fp = log file descriptor.
		s = socket descriptor.
		pdu = valid AccessControlRequest PDU structure.
POST:		On success, returns 1 and the PDU is sent.
		On failure, returns an error code and the PDU is not sent.
ERRORS:		See zEncodePDU()
*/
int fwAccessControlRequest(FILE *fp, NETBOXPROFILE*  s, PDU *pdu)
{
 	int result;
 
	result=encodeZ3950PDU(pdu,s);
	if(result!=1) {
		fwLog(fp,0,0,"access:fwAccessControlRequest():failed with %i",result);
		return result;
	}

	return 1;
}

/*
FUNCTION:	fwAccessControlResponse()
PURPOSE:	Sends an AccessControlResponse over the wire.
PRE:		fp = log file descriptor.
		s = socket descriptor.
		pdu = valid AccessControlResponse PDU structure.
POST:		On success, returns 1 and the PDU is sent.
		On failure, returns an error code and the PDU is not sent.
ERRORS:		See zEncodePDU()
*/
int fwAccessControlResponse(FILE *fp, NETBOXPROFILE* s, PDU *pdu) 
{
	int result;
 
	result=encodeZ3950PDU(pdu,s);
	if(result!=1) {
		fwLog(fp,0,0,"access:fwAccessControlResponse():failed with %i",result);
		return result;
	}
	
	return 1;
}

/*
FUNCTION:	fwBuildAccessControlRequest()
PURPOSE:	Allocates an AccessControlRequest PDU and fills with specified
		parameters.
PRE:		refId = reference Id
		request = String requesting information from client.	
POST:		On success, returns an AccessControlRequest PDU.
		On failure, returns NULL PDU pointer.
*/
PDU *fwBuildAccessControlRequest(CHAR *refId, CHAR *request)
{
	PPDU pdu;
 
	if((pdu = NEW(PDU)) == NULL)
		return NULL;
	pdu->which = t_accessControlRequest;
	pdu->u.accessControlRequest=NEW(ACCESSCONTROLREQUEST);
        if(*refId!=NULL)
		pdu->u.accessControlRequest->referenceId=
			encodeOctetStr(strdup(refId));
	/* We're only supporting old way now */
	pdu->u.accessControlRequest->which=t_rqoldWay;
        if(*request!=NULL)
		pdu->u.accessControlRequest->u.oldWay=
			encodeOctetStr(strdup(request));
 
	return pdu;
}

/*
FUNCTION:	fwBuildAccessControlResponse()
PURPOSE:	Allocates an AccessControlResponse PDU and fills with specified
		parameters.
PRE:		refId = Reference Id
		response = client's response to the server	
POST:		On success, returns an AccessControlResponse PDU.
		On failure, returns NULL PDU pointer.
*/
PDU *fwBuildAccessControlResponse(CHAR *refId, CHAR *response)
{
	PPDU pdu;

	if((pdu = NEW(PDU)) == NULL)
		return NULL;
	pdu->which = t_accessControlResponse;
	pdu->u.accessControlResponse=NEW(ACCESSCONTROLRESPONSE);
	if(*refId != NULL)
		pdu->u.accessControlResponse->referenceId=
			encodeOctetStr(strdup(refId));
	/* We only support old way now */
	pdu->u.accessControlResponse->which=t_rqoldWay;
	if(*response != NULL)
		pdu->u.accessControlResponse->u.oldWay=
			encodeOctetStr(strdup(response));

	return pdu;
}

/*
FUNCTION:	fwFreeAccessControlRequest()
PURPOSE:	Frees an allocated AccessControlRequest PDU structure.
PRE:		pdu = pointer to allocated PDU structure.
POST:		Memory allocated by pdu is freed.
*/
void fwFreeAccessControlRequest(PPDU pdu) 
{
	freeOctetString(pdu->u.accessControlRequest->referenceId);
	freeOctetString(pdu->u.accessControlRequest->u.oldWay);
	free(pdu->u.accessControlRequest);
	free(pdu);
}

/*
FUNCTION:	fwFreeAccessControlResponse()
PURPOSE:	Frees an allocated AccessControlResponse PDU structure.
PRE:		pdu = pointer to allocated PDU structure.
POST:		Memory allocated by pdu is freed.
*/
void fwFreeAccessControlResponse(PPDU pdu) 
{
	freeOctetString(pdu->u.accessControlResponse->referenceId);
	freeOctetString(pdu->u.accessControlResponse->u.oldWay);
	free(pdu->u.accessControlResponse);
	free(pdu);
}
