/***************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery
and Retrieval, 1994.

Permission to use, copy, modify, distribute, and sell this software
and its documentation, in whole or in part, for any purpose is hereby
granted without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included
in this distribution must remain intact.
2. Users of this software agree to make their best efforts (a) to
return to MCNC any improvements or extensions that they make, so that
these may be included in future releases; and (b) to inform MCNC/CNIDR
of noteworthy uses of this software.

3. The names of MCNC and Clearinghouse for Networked Information
Discovery and Retrieval may not be used in any advertising or publicity
relating to the software without the specific, prior written permission
of MCNC/CNIDR.

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

***************************************************************/
#include "platform.h"
#ifdef PLATFORM_MSDOS
#define MAXPROFILESTR 256
#ifdef PLATFORM_BORLANDC
#include <alloc.h>
#endif
#ifdef PLATFORM_MSC
#include <malloc.h>
#endif
#endif
#include <string.h>
#include <stdio.h>
#include "netbox.h"
#include "strbox.h"
#include "berutil.h"
#include "zdefs.h"
#include "init.h"
#include "search.h"
#include "present.h"
#include "zdecode.h"
#include "query.h"
#include "log.h"
#include "rpn.h"
#include "zutils.h"
#include "freemem.h"

#define OK 1
#define NOTOK -1

#include "freemem.h"

/*
FUNCTION:	freeOctetString()
PURPOSE:	Frees an OCTETSTRING structure.
PRE:		s = a previously allocated OCTETSTRING structure.
POST:		Memory used by s is freed.
*/
void freeOctetString(POCTETSTRING s)
{
	if(s!=NULL) {
		if(s->data!=NULL)
			free(s->data);
		free(s);
	}
}

/*
FUNCTION:	freeBitString()
PURPOSE:	Frees a BITSTRING structure.
PRE:		s = a previously allocated BITSTRING structure.
POST:		Memory used by s is freed.
*/
void freeBitString(POCTETSTRING s) {
	freeOctetString(s);
}

/*
FUNCTION:	freeObjectIdentifier()
PURPOSE:	Frees an OBJECTIDENTIFIER structure.
PRE:		s = a previously allocated OBJECTIDENTIFIER structure.
POST:		Memory used by s is freed.
*/
void freeObjectIdentifier(POBJECTIDENTIFIER s) {
	freeOctetString(s);
}

/*
FUNCTION:	freeAny()
PURPOSE:	Frees an ANY structure.
PRE:		a = a previously allocated ANY structure.
POST:		Memory used by a is freed.
*/
void freeAny(PANY a) {
	freePresentationElement(a);
}

/*
FUNCTION:	freePresentationElementList()
PURPOSE:	Frees a PRESENTATIONELEMENTLIST structure.
PRE:		p = a previously allocated PRESENTATIONELEMENTLIST structure.
POST:		Memory used by p is freed.
*/
void freePresentationElementList(PPRESENTATIONELEMENTLIST p) {
	if(p!=NULL) {
		if(p->next!=NULL)
			freePresentationElementList(p->next);
		freePresentationElement(p->item);
		free(p);
	} 
}

/*
FUNCTION:	freePresentationElement()
PURPOSE:	Frees a PRESENTATIONELEMENT structure.
PRE:		p = a previously allocated PRESENTATIONELEMENT structure.
POST:		Memory used by p is freed.
*/
void freePresentationElement(PPRESENTATIONELEMENT p) {
	if(p!=NULL) {
		if(p->PEForm > 0)
			freePresentationElementList(p->u.list);
		else
			freeOctetString(p->u.data);
		free(p);
	} 
}

/*
FUNCTION:	freeExternal()
PURPOSE:	Frees an EXTERNAL structure.
PRE:		e = a previously allocated EXTERNAL structure.
POST:		Memory used by e is freed.
*/
void freeExternal(PEXTERNAL e) {
	freePresentationElement(e);
}

/*
FUNCTION:	freeElementSetNames()
PURPOSE:	Frees an ELEMENTSETNAMES structure.
PRE:		e = a previously allocated ELEMENTSETNAMES structure.
POST:		Memory used by e is freed.
*/
void freeElementSetNames(PELEMENTSETNAMES e) {
	if(e!=NULL) {
		if(e->which==t_generic)
			freeOctetString(e->u.generic);
		else
			freeElementSetPlusDB(e->u.databaseSpecific);
		free(e);
	}
}

/*
FUNCTION:	freeQuery()
PURPOSE:	Frees a QUERY structure.
PRE:		q = a previously allocated QUERY structure.
POST:		Memory used by q is freed.
*/
void freeQuery(PQUERY q) {
	if(q!=NULL) {
		switch(q->which) {
			case t_type_0:
				freeAny(q->u.type_0);
				break;
			case t_type_1:
				freeRPNQuery(q->u.type_1); 
				break;
			case t_type_2:
				freeOctetString(q->u.type_2);
				break;
			case t_type_100:
				freeOctetString(q->u.type_100);
				break;
			case t_type_101:
				freeOctetString(q->u.type_101);
				break;
			default:
				break;
		} /* switch */
		free(q);
	}
}

/*
FUNCTION:	freeRPNQuery()
PURPOSE:	Frees an RPNQUERY structure.
PRE:		r = a previously allocated RPNQUERY structure.
POST:		Memory used by r is freed.
*/
void freeRPNQuery(PRPNQUERY r) {
	if(r!=NULL) {
		freeObjectIdentifier(r->attributeSetId);
		freeRPNStructure(r->rPNStructure);
		free(r);
	}
}

/*
FUNCTION:	freeRPNStructure()
PURPOSE:	Frees an RPNSTRUCTURE structure.
PRE:		r = a previously allocated RPNSTRUCTURE structure.
POST:		Memory used by r is freed.
*/
void freeRPNStructure(PRPNSTRUCTURE  r) {
	if(r!=NULL) {
		if(r->which==t_operand)
			freeOperand(r->u.operand);
		else {
			freeRPNStructure(r->u.binop.arg1); 
			freeRPNStructure(r->u.binop.arg2); 
			if(r->u.binop.opcode!=NULL)
				free(r->u.binop.opcode);
		}
		free(r);
	}
}

/*
FUNCTION:	freeOperand()
PURPOSE:	Frees an OPERAND structure.
PRE:		o = a previously allocated OPERAND structure.
POST:		Memory used by o is freed.
*/
void freeOperand(POPERAND o) {
	if(o!=NULL) {
		if(o->which==t_attributesPlusTerm)
			freeAttributesPlusTerm(o->u.attributesPlusTerm);
		else
			freeOctetString(o->u.resultSetId);
		free(o);
	}
}

/*
FUNCTION:	freeAttributesPlusTerm()
PURPOSE:	Frees an ATTRIBUTESPLUSTERM structure.
PRE:		a = a previously allocated ATTRIBUTESPLUSTERM structure.
POST:		Memory used by a is freed.
*/
void freeAttributesPlusTerm(PATTRIBUTESPLUSTERM a) {
	if(a!=NULL) {
		freeOctetString(a->term);
		freeAttributeList(a->attributeList);
		free(a);
	}
}

/*
FUNCTION:	freeAttributeList()
PURPOSE:	Frees an ATTRIBUTEELEMENTLIST structure.
PRE:		a = a previously allocated ATTRIBUTEELEMENTLIST structure.
POST:		Memory used by a is freed.
*/
void freeAttributeList(PATTRIBUTEELEMENTLIST a) {
	if(a!=NULL) {
		if(a->next!=NULL)
			freeAttributeList(a->next);
		free(a->item);
		free(a);
	}
}

/*
FUNCTION:	freeElementSetPlusDB()
PURPOSE:	Frees an ELEMENTSETPLUSDB structure.
PRE:		e = a previously allocated ELEMENTSETPLUSDB structure.
POST:		Memory used by a is freed.
*/
void freeElementSetPlusDB(PELEMENTSETPLUSDB e) {
	if(e!=NULL) {
		PELEMENTSETPLUSDB tmp;
		tmp=e;
		if(tmp->next!=NULL)
			freeElementSetPlusDB(tmp->next);
		freeOctetString(tmp->item.databaseName);
		freeOctetString(tmp->item.elementSetName);
		free(e);
	}
}

/*
FUNCTION:	freeDatabaseNames()
PURPOSE:	Frees a DATABASENAMELIST structure.
PRE:		s = a previously allocated DATABASENAMELIST structure.
POST:		Memory used by s is freed.
*/
void freeDatabaseNames(PDATABASENAMELIST s) {
	if(s!=NULL) {
		if(s->next!=NULL)
			freeDatabaseNames(s->next);
		freeOctetString(s->item);
		free(s);
	}	
}

/*
FUNCTION:	freeDiagRecord()
PURPOSE:	Frees a PDIAGRECORD structure.
PRE:		r = a previously PDIAGRECORD structure.
POST:		Memory used by r is freed.
*/
void freeDiagRecord(PDIAGRECORD r) {

	freeObjectIdentifier(r->diagnosticSetId);
	freeOctetString(r->addinfo);
	free(r);		
}

/*
FUNCTION:	freeRecords()
PURPOSE:	Frees a RECORDS structure.
PRE:		r = a previously allocated RECORDS structure.
POST:		Memory used by r is freed.
*/
void freeRecords(PRECORDS r) {
	if(r!=NULL) {
		if(r->which==t_dataBaseOrSurDiagnostics) {
			PNAMEPLUSRECORDLIST nprList;
			nprList=r->u.dataBaseOrSurDiagnostics;
			while(nprList!=NULL) {
				PNAMEPLUSRECORD npr;
				npr=nprList->item;
	                        freeOctetString(npr->databaseName);
				if(npr->recordOrSurrogate.which == t_databaseRecord)
	                        	freePresentationElement(npr->recordOrSurrogate.u.databaseRecord);
				else {
					freeDiagRecord(npr->recordOrSurrogate.u.surrogateDiagnostic);                         	
				}
				nprList=nprList->next;                	
	                }
		}
		else {
			/* nonSurrogateDiagnostic */
			freeDiagRecord(r->u.nonSurrogateDiagnostic);
		}
	        free(r);
	}
}