/*************************************************************************

Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery and
Retrieval, 1994. 

Permission to use, copy, modify, distribute, and sell this software and
its documentation, in whole or in part, for any purpose is hereby granted
without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included in
this distribution must remain intact. 

2. Users of this software agree to make their best efforts (a) to return
to MCNC any improvements or extensions that they make, so that these may
be included in future releases; and (b) to inform MCNC/CNIDR of noteworthy
uses of this software. 

3. The names of MCNC and Clearinghouse for Networked Information Discovery
and Retrieval may not be used in any advertising or publicity relating to
the software without the specific, prior written permission of MCNC/CNIDR. 

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR
IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 

*************************************************************************/

#include "imem.h"

/* Private functions */

char FAR* AllocateMem(DWORD size, int Zero, int Realloc, char FAR* ptr);

/* Public functions */

char FAR* imem_malloc(DWORD size) {
	return AllocateMem(size, 0, 0, 0);
}

char FAR* imem_calloc(DWORD nelem, DWORD elsize) {
	return AllocateMem((nelem*elsize), 1, 0, 0);
}

char FAR* imem_realloc(void FAR* ptr, DWORD size) {
	return AllocateMem(size, 0, 1, ptr);
}

void imem_free(void FAR* ptr) {
#ifdef PLATFORM_WINDOWS
	GlobalUnlock( (HGLOBAL)LOWORD(GlobalHandle(SELECTOROF(ptr))) );
	GlobalFree( (HGLOBAL)LOWORD(GlobalHandle(SELECTOROF(ptr))) );
#else
	free(ptr);
#endif
}

/* Private functions */

char FAR* AllocateMem(DWORD size, int Zero, int Realloc, char FAR* ptr) {
#ifdef PLATFORM_WINDOWS
	HGLOBAL hg;
	UINT fuAlloc;
	void FAR* p;
	if (Zero)
		fuAlloc = GMEM_MOVEABLE | GMEM_ZEROINIT;
	else
		fuAlloc = GMEM_MOVEABLE;
	if (Realloc)
		hg = GlobalReAlloc( \
			(HGLOBAL)LOWORD(GlobalHandle(SELECTOROF(ptr))), \
			(DWORD)size, fuAlloc);
	else
		hg = GlobalAlloc(fuAlloc, size);
	if (hg == NULL)
		return NULL;
	p = GlobalLock(hg);
	if (p == NULL) {
		GlobalFree(hg);
		return NULL;
	}
	return (char FAR*)p;
#else
	if (Realloc) {
		return (char*)realloc(ptr, size);
	} else {
		if (Zero)
			return (char*)calloc(1, size);
		else
			return (char*)malloc(size);
	}
#endif
}
