/***************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery
and Retrieval, 1994.

Permission to use, copy, modify, distribute, and sell this software
and its documentation, in whole or in part, for any purpose is hereby
granted without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included
in this distribution must remain intact.
2. Users of this software agree to make their best efforts (a) to
return to MCNC any improvements or extensions that they make, so that
these may be included in future releases; and (b) to inform MCNC/CNIDR
of noteworthy uses of this software.

3. The names of MCNC and Clearinghouse for Networked Information
Discovery and Retrieval may not be used in any advertising or publicity
relating to the software without the specific, prior written permission
of MCNC/CNIDR.

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

***************************************************************/
#include "platform.h"
#ifdef PLATFORM_MSDOS
#define MAXPROFILESTR 256
#ifdef PLATFORM_BORLANDC
#include <alloc.h>
#endif
#ifdef PLATFORM_MSC
#include <malloc.h>
#endif
#endif
#include <string.h>
#include <stdio.h>
#include "netbox.h"
#include "strbox.h"
#include "berutil.h"
#include "zdefs.h"
#include "init.h"
#include "search.h"
#include "present.h"
#include "zdecode.h"
#include "query.h"
#include "ztags.h"
#include "rpn.h"
#include "zutils.h"

#define TRUE 1
#define FALSE 0
#define OK 1
#define NOTOK -1

#ifdef ANSI_LIKE
#include <stdarg.h>
#else
#include <varargs.h>
#endif
#include "log.h"

void 
#ifdef ANSI_LIKE
fwLog(FILE *pLogFile, long pLevel1, long pLevel2, char *pFormat, ...)
#else
fwLog(va_alist) 
va_dcl 
#endif
{
	va_list ap;

#ifndef ANSI_LIKE
	long lLevel1, lLevel2;
	CHAR *lFormat;
	FILE *lLogFile;

	va_start(ap);
	lLogFile   = va_arg(ap, FILE *);
	lLevel1 = va_arg(ap, long);
	lLevel2 = va_arg(ap, long);
	lFormat = va_arg(ap, CHAR *);
  
	if(lLogFile!=NULL) {
		/* Write to log file */
		vfprintf(lLogFile,lFormat, ap);
		fprintf(lLogFile,"\n");
		fflush(lLogFile);
	}

	va_end(ap);
#else 
	va_start(ap, pFormat);
  
	if(pLogFile!=NULL) {
	/* Write to log file */
		vfprintf(pLogFile,pFormat, ap);
		fprintf(pLogFile,"\n");
		fflush(pLogFile);
	}

	va_end(ap);
#endif /* ifndef ANSI_LIKE */

}

void fwLogPDU(FILE *fp,PPDU pdu) {
	CHAR *tmp;
	CHAR *buf=(CHAR *)calloc(1,64);

	if(fp==NULL || pdu==NULL)
		return;

	theTime(buf);
	fwLog(fp,0,0,"%s",buf);
	free(buf);

	switch(pdu->which) {
		case t_initRequest: {
			PINITIALIZEREQUEST IR;
			IR=pdu->u.initRequest;
			fwLog(fp,0,0,"InitRequest {");

			if(IR->referenceId!=NULL) {
				fwLogOString(fp,"  ReferenceId=",
				IR->referenceId);
			} else fwLog(fp,0,0,"  ReferenceId=Unused");
      
			if(IR->protocolVersion!=NULL) {
				fwLog(fp,0,0,"  ProtocolVersion=%X%X",
				IR->protocolVersion->data[0],
				IR->protocolVersion->data[1]);
			} else fwLog(fp,0,0,"  ProtocolVersion=Unused");

			if(IR->idAuthentication!=NULL) {
				fwLogOString(fp,"  IdAuthentication=",
				IR->idAuthentication->u.data);
			} else
				fwLog(fp,0,0,"  IdAuthentication=Unused");

			if(IR->options!=NULL) {
				fwLog(fp,0,0,"  Options=%X%X",
				IR->options->data[0],
				IR->options->data[1]);
			} else fwLog(fp,0,0,"  Options=Unused");

			fwLog(fp,0,0,"  PreferredMessageSize=%ld",
			IR->preferredMessageSize);

			fwLog(fp,0,0,"  MaximumRecordSize=%ld",
			IR->maximumRecordSize);

			if(IR->implementationId!=NULL) {
				fwLogOString(fp,"  ImplementationId=",
				IR->implementationId);
			} else fwLog(fp,0,0,"  ImplementationId=Unused");

			if(IR->implementationName!=NULL) {
				fwLogOString(fp,"  ImplementationName=",
				IR->implementationName);
			} else fwLog(fp,0,0,"  ImplementationName=Unused");

			if(IR->implementationVersion!=NULL) {
				fwLogOString(fp,"  ImplementationVersion=",
				IR->implementationVersion);
			} else fwLog(fp,0,0,"  ImplementationVersion=Unused");

			fwLog(fp,0,0,"  UserInformationField=Not logging yet.");
      
			fwLog(fp,0,0,"}");
			break;
		}
		case t_initResponse: {
			PINITIALIZERESPONSE IR;
			IR=pdu->u.initResponse;
			fwLog(fp,0,0,"InitResponse {");

			if(IR->referenceId!=NULL) {
				fwLogOString(fp,"  ReferenceId=",
				IR->referenceId);
			} else fwLog(fp,0,0,"  ReferenceId=Unused");
      
			if(IR->protocolVersion!=NULL) {
				fwLog(fp,0,0,"  ProtocolVersion=%X%X",
				IR->protocolVersion->data[0],
				IR->protocolVersion->data[1]);
			} else fwLog(fp,0,0,"  ProtocolVersion=Unused");

			if(IR->result>0)
				fwLog(fp,0,0,"  Result=ACCEPTED");
			else
				fwLog(fp,0,0,"  Result=DENIED");

			if(IR->options!=NULL) {
				unsigned char b1,b2;
				b1=IR->options->data[0];
				b2=IR->options->data[1];
				fwLog(fp,0,0,"  Options=%X%X",b1,b2);
			} else fwLog(fp,0,0,"  Options=Unused");

			fwLog(fp,0,0,"  PreferredMessageSize=%ld",
				IR->preferredMessageSize);

			fwLog(fp,0,0,"  MaximumRecordSize=%ld",
				IR->maximumRecordSize);

			if(IR->implementationId!=NULL) {
				fwLogOString(fp,"  ImplementationId=",
				IR->implementationId);
			} else fwLog(fp,0,0,"  ImplementationId=Unused");

			if(IR->implementationName!=NULL) {
				fwLogOString(fp,"  ImplementationName=",
				IR->implementationName);
			} else fwLog(fp,0,0,"  ImplementationName=Unused");

			if(IR->implementationVersion!=NULL) {
				fwLogOString(fp,"  ImplementationVersion=",
				IR->implementationVersion);
			} else fwLog(fp,0,0,"  ImplementationVersion=Unused");

			fwLog(fp,0,0,"  UserInformationField=Not Logging Yet.");
  
			fwLog(fp,0,0,"}");
			break;
		}    
		case t_searchRequest: {
			PSEARCHREQUEST SR;
                        SR=pdu->u.searchRequest;
			fwLog(fp,0,0,"SearchRequest {");

			if(SR->referenceId!=NULL) {
				fwLogOString(fp,"  ReferenceId=",
				SR->referenceId);
			} else fwLog(fp,0,0,"  ReferenceId=Unused");

			fwLog(fp,0,0,"  SmallSetUpperBound=%ld",
				SR->smallSetUpperBound);

			fwLog(fp,0,0,"  LargeSetLowerBound=%ld",
				SR->largeSetLowerBound);
  
			fwLog(fp,0,0,"  MediumSetPresentNumber=%ld",
				SR->mediumSetPresentNumber);
 
			if(SR->replaceIndicator==TRUE)
				fwLog(fp,0,0,"  ReplaceSetIndicator=TRUE");
			else
				fwLog(fp,0,0,"  ReplaceSetIndicator=FALSE");
     
			if(SR->resultSetName!=NULL) {
				fwLogOString(fp,"  ResultSetName=",
				SR->resultSetName);
			} else fwLog(fp,0,0,"  ResultSetName=Unused");
  
			if(SR->databaseNames!=NULL) {
				PDATABASENAMELIST lSrPtr;
				lSrPtr=SR->databaseNames;
				while(lSrPtr!=NULL) {
					fwLogOString(fp,"  DatabaseName=",
						lSrPtr->item);
					lSrPtr=lSrPtr->next;
				}
			} else fwLog(fp,0,0,"  DatabaseName=Unused");

			if(SR->smallSetElementSetNames!=NULL) {
				fwLog(fp,0,0,"  SmallSetElementSetNames {");
				fwLogElementSetNames(fp,
				SR->smallSetElementSetNames);
				fwLog(fp,0,0,"   }");
			} else fwLog(fp,0,0,"  SmallSetElementSetNames=Unused");
   
			if(SR->mediumSetElementSetNames!=NULL) {
				fwLog(fp,0,0,"  MediumSetElementSetNames {");
				fwLogElementSetNames(fp,
				SR->mediumSetElementSetNames);
				fwLog(fp,0,0,"   }");
			} else fwLog(fp,0,0,"  MediumSetElementSetNames=Unused");
  
			if(SR->preferredRecordSyntax!=NULL)
				fwLogOString(fp,"  PreferredRecordSyntax=",
				SR->preferredRecordSyntax);
			else fwLog(fp,0,0,"  PreferredRecordSyntax=Unused");

			if(SR->query!=NULL) {
				fwLog(fp,0,0,"  Query {");
				fwLogQuery(fp,"  Query=",
				SR->query);
				fwLog(fp,0,0,"  }");
			} else fwLog(fp,0,0,"  Query=Unused");

			fwLog(fp,0,0,"}");

			break;
		}

		case t_searchResponse: {
			PSEARCHRESPONSE SR;
                        SR=pdu->u.searchResponse;
			fwLog(fp,0,0,"SearchResponse {");

			if(SR->referenceId!=NULL) {
				fwLogOString(fp,"  ReferenceId=",
				SR->referenceId);
			} else fwLog(fp,0,0,"  ReferenceId=Unused");

			fwLog(fp,0,0,"  ResultCount=%ld",
				SR->resultCount);
      
			fwLog(fp,0,0,"  NumberOfRecordsReturned=%ld",
				SR->numberOfRecordsReturned);

			fwLog(fp,0,0,"  NextResultSetPosition=%ld",
				SR->nextResultSetPosition);

			if(SR->searchStatus>0)
				fwLog(fp,0,0,"  SearchStatus=TRUE");
			else
				fwLog(fp,0,0,"  SearchStatus=FALSE");

			if(SR->resultSetStatus>0)
				fwLog(fp,0,0,"  ResultSetStatus=TRUE");
			else
				fwLog(fp,0,0,"  ResultSetStatus=FALSE");

			if(SR->presentStatus>0)
				fwLog(fp,0,0,"  PresentStatus=FALSE");
			else
				fwLog(fp,0,0,"  PresentStatus=TRUE");

			fwLog(fp,0,0,"  DatabaseOrDiagnosticRecords {");
			if(SR->databaseOrDiagnosticRecords!=NULL)
				fwLogRecords(fp,SR->databaseOrDiagnosticRecords);
			else fwLog(fp,0,0,"    No Records Returned");
				fwLog(fp,0,0,"  }");
      
			fwLog(fp,0,0,"}");
			break;
		}
		case t_presentRequest: {
			PPRESENTREQUEST PR;
                        PR=pdu->u.presentRequest;
			fwLog(fp,0,0,"PresentRequest {");

			if(PR->referenceId!=NULL) {
				fwLogOString(fp,"  ReferenceId=",
				PR->referenceId);
			} else fwLog(fp,0,0,"   ReferenceId=Unused");

			if(PR->resultSetId!=NULL) {
				fwLogOString(fp,"   ResultSetId=",
				PR->resultSetId);
			} else fwLog(fp,0,0,"   ResultSetId=");

			fwLog(fp,0,0,"   ResultSetStartPoint=%ld",
				PR->resultSetStartPoint);

			fwLog(fp,0,0,"   NumberOfRecordsRequested=%ld",
				PR->numberOfRecordsRequested);
      
			fwLog(fp,0,0,"   ElementSetNames {");
			if(PR->elementSetNames!=NULL)
				fwLogElementSetNames(fp,PR->elementSetNames);
			fwLog(fp,0,0,"   }");
  
			if(PR->preferredRecordSyntax!=NULL)
				fwLogOString(fp,"  PreferredRecordSyntax=",
				PR->preferredRecordSyntax);
			else fwLog(fp,0,0,"  PreferredRecordSyntax=Unused");

			fwLog(fp,0,0,"}");
			break;
		}
		case t_presentResponse: {
			PPRESENTRESPONSE PR;
			PR=pdu->u.presentResponse;
			fwLog(fp,0,0,"presentResponse {");

			if(PR->referenceId!=NULL) {
				fwLogOString(fp, "  ResponseId=",
				PR->referenceId);
			} else fwLog(fp,0,0,"   ReferenceId=Unused");

			fwLog(fp,0,0,"   NumberOfRecordsReturned=%ld",
				PR->numberOfRecordsReturned);

			fwLog(fp,0,0,"   NextResultSetPosition=%ld",
				PR->nextResultSetPosition);
      
			if(PR->presentStatus>0)
				fwLog(fp,0,0,"   PresentStatus=FALSE");
			else
				fwLog(fp,0,0,"   PresentStatus=TRUE");
      
			if(PR->numberOfRecordsReturned>0)
				fwLogRecords(fp,
					PR->databaseOrDiagnosticRecords);
            
			fwLog(fp,0,0,"}");
			break;
		}

		default:
			fwLog(fp,0,0,"  Unsupported PDU type!Type=%ld",
				pdu->which);
	}
}

void fwLogQuery(FILE *fp,char *identifier, PQUERY query) {

	if(query!=NULL) {
		switch(query->which) {
			case t_type_0:
				fwLog(fp,0,0,"    Type=TYPE_0");
				break;
			case t_type_1:
				fwLog(fp,0,0,"    Type=TYPE_1");
				fwLog(fp,0,0,"    RPNQuery {");
				fwLogRPNQuery(fp,query->u.type_1);
				fwLog(fp,0,0,"    }");
				break;
			case t_type_2:
				fwLog(fp,0,0,"    Type=TYPE_2");
				break;
			case t_type_100:
				fwLog(fp,0,0,"    Type=TYPE_100");
				break;
			case t_type_101:
				fwLog(fp,0,0,"    Type=TYPE_101");
				break;
			default:
				fwLog(fp,0,0,"    Type=UNDOCUMENTED");
				break;
		}
	}
}

void fwLogRPNQuery(FILE *fp,PRPNQUERY rpn) {

	if(rpn->rPNStructure!=NULL)
		fwLogOString(fp,"    AttributeSetId=",
			rpn->attributeSetId);
	else fwLog(fp,0,0,"    AttributeSetId=Unused");

	if(rpn->rPNStructure!=NULL) {
		fwLog(fp,0,0,"      RPNStructure {");
		fwLogRPNStructure(fp,rpn->rPNStructure);  
		fwLog(fp,0,0,"      }");
	} else fwLog(fp,0,0,"      RPNStructure=Unused");
}

void fwLogRPNStructure(FILE *fp,PRPNSTRUCTURE rpns) {

	if(rpns->which==t_operand) {
		fwLog(fp,0,0,"        Operand {");
		if(rpns->u.operand->which == t_attributesPlusTerm) {
			PATTRIBUTESPLUSTERM attrsTerm;
			if(rpns->u.operand->u.attributesPlusTerm!=NULL) {
				attrsTerm=rpns->u.operand->u.attributesPlusTerm;
				fwLog(fp,0,0,"          AttributesPlusTerm {");
				if(attrsTerm->term!=NULL) 
					fwLogOString(fp,"            Term=",
						attrsTerm->term); 
				else fwLog(fp,0,0,"            Term=Unused");
				fwLogAttributeList(fp,attrsTerm->attributeList);
				fwLog(fp,0,0,"          }");
			} else fwLog(fp,0,0,"        AttributesPlusTerm=Unused");
		} else {
			if(rpns->u.operand->u.resultSetId!=NULL)
				fwLogOString(fp,"        ResultSetId=",
					rpns->u.operand->u.resultSetId);
			else fwLog(fp,0,0,"        ResultSetId=Unused");
		}
		fwLog(fp,0,0,"        }");
	} else {
		/* Not operand, but binop */
		fwLog(fp,0,0,"        Binop {");
		fwLogRPNStructure(fp,rpns->u.binop.arg1);
		fwLogRPNStructure(fp,rpns->u.binop.arg2);
		switch(rpns->u.binop.opcode->which) {
			case t_and:
				fwLog(fp,0,0,"        Operator=AND");
				break;
			case t_or:
				fwLog(fp,0,0,"        Operator=OR");
				break;
			case t_andNot:
				fwLog(fp,0,0,"        Operator=AND-NOT");
				break;
			default:
				fwLog(fp,0,0,"        Operator=UNDOCUMENTED");
		} 
	}
}

void fwLogAttributeList(FILE *fp,PATTRIBUTEELEMENTLIST attrList) {

	if(attrList!=NULL) {
		PATTRIBUTEELEMENT attrElement;
		fwLog(fp,0,0,"            Attributes {");
		if(attrList->item!=NULL) {
			if(attrList->item->attributeType!= 0 &&
				attrList->item->attributeValue != 0) {
				while(attrList!=NULL) {
					if(attrList->item!=NULL) {
						fwLog(fp,0,0,
						"              Type=%ld",
						attrList->item->attributeType); 
						fwLog(fp,0,0,
						"              Value=%ld",
						attrList->item->attributeValue); 
					} else fwLog(fp,0,0,
					"              AttributeElement=Unused");
					attrList=attrList->next;
				} /* while */
			} else fwLog(fp,0,0,
				"              EMPTY");
		} /* if */
	} else fwLog(fp,0,0,"          Attributes=Unused");
		fwLog(fp,0,0,"            }");
}


void fwLogOString(FILE *fp,char *identifier,POCTETSTRING s) {
 CHAR *tmp;

	if(s!=NULL) {
		if(s->length>0) {
			if((tmp=decodeOctetStr(s))!=NULL) {
				if(identifier!=NULL)
					fwLog(fp,0,0,"%s%s",identifier,tmp);
				else
					fwLog(fp,0,0,"%s",tmp);
				free(tmp);
			}
		} else fwLog(fp,0,0,"%s=Unused",identifier);
	} else fwLog(fp,0,0,"%s=Unused",identifier);
}

void fwLogRecords(FILE *fp,PRECORDS pRecords) {
	if(pRecords!=NULL) {
		switch(pRecords->which) {
			case t_dataBaseOrSurDiagnostics: {
				PNAMEPLUSRECORDLIST nprList;
				fwLog(fp,0,0,"    DatabaseOrSurrogateDiagnostics {");
				nprList=pRecords->u.dataBaseOrSurDiagnostics;
				while(nprList!=NULL) {
					fwLogNamePlusRecord(fp,nprList->item);
					nprList=nprList->next;
				}
				fwLog(fp,0,0,"    }");
				break;
			}
			case t_nonSurrogateDiagnostic: {
				PDIAGRECORD DR;
				DR=pRecords->u.nonSurrogateDiagnostic;
				fwLog(fp,0,0,"      NonSurrogateDiagnosticRecord {");
				fwLog(fp,0,0,"        DiagnosticSetId=%s",
				decodeOctetStr(DR->diagnosticSetId));
				fwLog(fp,0,0,"        Condition=%ld",DR->condition);
				fwLog(fp,0,0,"        AddInfo=%s",decodeOctetStr(DR->addinfo));
				fwLog(fp,0,0,"      }");
				break;
			}
			default:
				fwLog(fp,0,0,"      Unknown Record Type {");
				fwLog(fp,0,0,"      }");
				break;
		} /* switch */
	} else fwLog(fp,0,0,"  Records=Unused");
}

void fwLogNamePlusRecord(FILE *fp, PNAMEPLUSRECORD npr) {

        if(npr!=NULL) {
		if(npr->databaseName!=NULL) {
			fwLog(fp,0,0,"      DatabaseName=%s",
			decodeOctetStr(npr->databaseName));
		} else fwLog(fp,0,0,"     DatabaseName=Unused");
		switch(npr->recordOrSurrogate.which) {
			case t_databaseRecord: {
				if(npr->recordOrSurrogate.u.databaseRecord!=NULL) {
					fwLog(fp,0,0,"      DatabaseRecord {");
					fwLogPE(fp,
					npr->recordOrSurrogate.u.databaseRecord);
					fwLog(fp,0,0,"      }");
				} else fwLog(fp,0,0,"      DatabaseRecord=Unused");
				break;
			}
			case t_surrogateDiagnostic: {
					PDIAGRECORD DR;
					DR=npr->recordOrSurrogate.u.surrogateDiagnostic;
					fwLog(fp,0,0,"      SurrogateDiagnosticRecord {");
					fwLog(fp,0,0,"        DiagnosticSetId=%s",
					decodeOctetStr(DR->diagnosticSetId));
					fwLog(fp,0,0,"        Condition=%ld",DR->condition);
					fwLog(fp,0,0,"        AddInfo=%s",decodeOctetStr(DR->addinfo));
					fwLog(fp,0,0,"      }");
					break;
			}
			default:
				fwLog(fp,0,0,"      Unknown Record Type {");
				fwLog(fp,0,0,"      }");
				break;
		}
	} /* if(npr) */
}

void fwLogPE(FILE *fp,PPRESENTATIONELEMENT tmp) {

	if(tmp->PEForm > 0) {
		fwLog(fp,0,0,"Form=%ld",tmp->PEForm);
		fwLog(fp,0,0,"Tag=%ld",tmp->PETag);
		fwLogPEList(fp,tmp->u.list);
	} else {
		fwLog(fp,0,0,"Form=%ld",tmp->PEForm);
		fwLog(fp,0,0,"Tag=%ld",tmp->PETag);
		fwLogOString(fp,"",tmp->u.data);
	}
}

void fwLogPEList(FILE *fp,PPRESENTATIONELEMENTLIST peList) {
	while(peList!=NULL) {
		fwLogPE(fp,peList->item);
		peList=peList->next;
	}
}

void fwLogElementSetNames(FILE *fp,PELEMENTSETNAMES pEsnPtr) {

	switch(pEsnPtr->which) {
		case t_generic:
			fwLogOString(fp,"      Generic=",pEsnPtr->u.generic);
			break;
		case t_databaseSpecific: {
			PELEMENTSETPLUSDB ESDB;
			fwLog(fp,0,0,"      DatabaseSpecific {");
			ESDB=pEsnPtr->u.databaseSpecific;
			while(ESDB!=NULL) {
				fwLogOString(fp,"         DatabaseName=",
				ESDB->item.databaseName);
				fwLogOString(fp,"         ElementSetName=",
				ESDB->item.elementSetName);
				ESDB=ESDB->next;
			}
			fwLog(fp,0,0,"      }");
			break;
		}
		default:
			fwLog(fp,0,0,"      Unknown Element Set Type");
	} 
}
