/*************************************************************************

Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery and
Retrieval, 1994. 

Permission to use, copy, modify, distribute, and sell this software and
its documentation, in whole or in part, for any purpose is hereby granted
without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included in
this distribution must remain intact. 

2. Users of this software agree to make their best efforts (a) to return
to MCNC any improvements or extensions that they make, so that these may
be included in future releases; and (b) to inform MCNC/CNIDR of noteworthy
uses of this software. 

3. The names of MCNC and Clearinghouse for Networked Information Discovery
and Retrieval may not be used in any advertising or publicity relating to
the software without the specific, prior written permission of MCNC/CNIDR. 

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR
IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 

*************************************************************************/

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include "netbox.h"

#ifdef PLATFORM_WINDOWS
#include <sys\types.h>
#else
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/time.h>
#endif

int netbox_WSAStartup(void) {
#ifdef PLATFORM_WINDOWS
	WSADATA WsaData;
	if ( WSAStartup(0x0101, &WsaData) != 0 )
		return -1;
#endif
	return 0;
}

void netbox_WSACleanup(void) {
#ifdef PLATFORM_WINDOWS
	WSACleanup();
#endif
}

int netbox_SetBufferSize(NETBOXPROFILE* profile, int BufferSize) {
	int BufSize = BufferSize;
	return setsockopt(profile->Socket, SOL_SOCKET, SO_RCVBUF,
			(CHAR FAR*)&BufSize, sizeof(BufSize));
}

int netbox_Open(NETBOXPROFILE* profile) {
	SOCKET s;
	CHAR HostNameBuf[256];
	LPHOSTENT HostEnt;
	SOCKADDR_IN addr_in;
	int rc;
	int BufSize;
	memset((CHAR*)&addr_in, 0, sizeof(addr_in));
	addr_in.sin_addr.s_addr = inet_addr(profile->HostName);
	if (addr_in.sin_addr.s_addr != -1) {
		addr_in.sin_family = AF_INET;
		strcpy(HostNameBuf, profile->HostName);
	} else {
		HostEnt = gethostbyname(profile->HostName);
		if ( NULL == HostEnt ) {
			return 0;
		}
		addr_in.sin_family = HostEnt->h_addrtype;
#ifdef h_addr
		strncpy((CHAR*)&addr_in.sin_addr, HostEnt->h_addr_list[0],
				HostEnt->h_length);
#endif
		strcpy(HostNameBuf, HostEnt->h_name);
	}
	addr_in.sin_port = htons(profile->Port);
	if ( (s=socket(AF_INET, SOCK_STREAM, 0)) == INVALID_SOCKET )
		return 0;
	rc = connect(s, (struct sockaddr FAR*)&addr_in, sizeof(addr_in));
	if (rc == -1)
		return 0;
	profile->Socket = s;
	return 1;
}

int netbox_Close(NETBOXPROFILE* profile) {
	if ( closesocket(profile->Socket) == 0 ) {
		netbox_WSACleanup();
		return 1;
	} else {
		return 0;
	}
}

int netbox_SendBuffer(NETBOXPROFILE* profile, CHAR FAR* Buffer,
		int BufferSize) {
	/* select() checking should be inserted here */
#ifdef PLATFORM_WINDOWS
	if ( send(profile->Socket, Buffer, BufferSize, 0) == SOCKET_ERROR )
		return -1;
#else
	if ( write(profile->Socket, Buffer, BufferSize) == -1 ) /* == ERROR? */
		return -1;
#endif
	return BufferSize;
}

int netbox_iGetBuffer(NETBOXPROFILE* profile, CHAR FAR* Buffer,
		int BufferSize) {
	/* select() checking should be inserted here */
	int x;
	int TotalRead = 0;
	int Left;
	CHAR* p;
	p = Buffer;
	Left = BufferSize;
	do {
#ifdef PLATFORM_WINDOWS
		x = recv(profile->Socket, p, Left, 0);
#else
		x = read(profile->Socket, p, Left);
#endif
		if ( (x != SOCKET_ERROR) && (x != 0) ) {
			p += x;
			Left -= x;
			TotalRead += x;
		}
	} while ( (x != SOCKET_ERROR) && (x != 0) &&
			(TotalRead < BufferSize) );
	if (x == SOCKET_ERROR)
		return -1;
	else
		return TotalRead;
}

long netbox_GetBuffer(NETBOXPROFILE* profile, CHAR FAR* Buffer,
		long BufferSize) {
	int x = 1;
	long y;
	CHAR* p = Buffer;
	long Left = BufferSize;
	long TotalRead = 0;
	while ( (x != 0) && (TotalRead < BufferSize) ) {
		if (Left > MAXINT)
			y = MAXINT;
		else
			y = Left;
		x = netbox_iGetBuffer(profile, p, y);
		if (x == -1)
			return -1;
		if (x != 0) {
			TotalRead += x;
			Left -= x;
			p += x;
		}
	}
	return TotalRead;
}

int netbox_WaitForData(NETBOXPROFILE* profile) {
	fd_set ReadFDS;
	struct timeval TimeVal;
	TimeVal.tv_sec = profile->TimeOutSec;
	TimeVal.tv_usec = profile->TimeOutUSec;
	FD_ZERO(&ReadFDS);
	FD_SET(profile->Socket, &ReadFDS);
	/* Check the socket for readability. */
	if ( (select((profile->Socket)+1, &ReadFDS, 0, 0, &TimeVal)) <= 0 ) {
		/* Timed out or SOCKET_ERROR! Not ready to write. */
		return 0;
	}
	return 1;
}

/*
SOCKET netbox_StartServer(CHAR FAR* HostName, int Port) {
	SOCKET s;
	SOCKADDR_IN addr_in;
	memset(&addr_in, 0, sizeof(addr_in));
	if ( (s=socket(AF_INET, SOCK_STREAM, 0)) == INVALID_SOCKET )
		return -1;
	addr_in.sin_family = AF_INET;
	addr_in.sin_addr.s_addr = htonl(INADDR_ANY);
	addr_in.sin_port = (unsigned short int)htons(Port);
	if ( bind(s, (struct sockaddr*)&addr_in, sizeof(addr_in)) ==
			SOCKET_ERROR ) {
#ifdef PLATFORM_WINDOWS
		if (WSAGetLastError() == WSAEADDRINUSE) {
			if ( connect(s, (const struct sockaddr FAR*)&addr_in,
					sizeof(addr_in)) == 0 ) {
				netbox_WSACleanup();
#else
		if (errno == EADDRINUSE) {
			if ( connect(s, (struct sockaddr*)&addr_in,
					sizeof(addr_in)) == 0 ) {
#endif
				close(s);
				return -1;
			}
		} else {
			int test = 1;
#ifdef PLATFORM_WINDOWS
			netbox_WSACleanup();
#endif
			close(s);
			if ( (s=socket(AF_INET, SOCK_STREAM, 0)) ==
					INVALID_SOCKET )
				return -1;
			setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &test,
					sizeof(test));
			addr_in.sin_family = AF_INET;
			addr_in.sin_addr.s_addr = INADDR_ANY;
			addr_in.sin_port = (unsigned short int)htons(Port);
			bind(s, addr_in, sizeof(addr_in));
		}
	}
	if ( listen(s, 8) == -1 ) {
#ifdef PLATFORM_WINDOWS
		netbox_WSACleanup();
#endif
		close(s);
		return -1;
	}
	return s;
}

int netbox_AcceptClient(SOCKET s) {
	SOCKADDR_IN addr_in;
	SOCKET NewSocket;
	do {
		if ( (NewSocket = accept(s, &addr_in, sizeof(addr_in))) ==
				INVALID_SOCKET)
			return -1;
#ifdef PLATFORM_WINDOWS
	} while (WSAGetLastError() == WSAEINTR);
#else
	} while (errno == EINTR);
#endif
	return NewSocket;		
}
*/
