/***************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery
and Retrieval, 1994.

Permission to use, copy, modify, distribute, and sell this software
and its documentation, in whole or in part, for any purpose is hereby
granted without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included
in this distribution must remain intact.
2. Users of this software agree to make their best efforts (a) to
return to MCNC any improvements or extensions that they make, so that
these may be included in future releases; and (b) to inform MCNC/CNIDR
of noteworthy uses of this software.

3. The names of MCNC and Clearinghouse for Networked Information
Discovery and Retrieval may not be used in any advertising or publicity
relating to the software without the specific, prior written permission
of MCNC/CNIDR.

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

***************************************************************/
#include "platform.h"
#ifdef PLATFORM_MSDOS
#define MAXPROFILESTR 256
#ifdef PLATFORM_BORLANDC
#include <alloc.h>
#endif
#ifdef PLATFORM_MSC
#include <malloc.h>
#endif
#endif
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "netbox.h"
#include "strbox.h"
#include "berutil.h"
#include "zdefs.h"
#include "init.h"
#include "search.h"
#include "present.h"
#include "zdecode.h"
#include "query.h"
#include "log.h"
#include "rpn.h"
#include "zutils.h"
#include "freemem.h"

#define NEW(a) (a*)calloc(1,sizeof(a));
#define OK 1
#define NOTOK -1
#define TRUE 1
#define FALSE 0
#include "rpn.h"

/* Local functions */
PRPNSTRUCTURE buildRPNStruct(CHAR *s);
CHAR *getArg(CHAR *s, int opcode,int argNum);
int binop(CHAR *s);
POPERAND buildOperand(CHAR *s);
PATTRIBUTESPLUSTERM buildAttrPlusTerm(CHAR *s);
void fwFreeRPNStructure(PRPNSTRUCTURE r);
void fwFreeRPNOperand(POPERAND o);
void fwFreeAttributesPlusTerm(PATTRIBUTESPLUSTERM a);
void fwFreeAttributeList(PATTRIBUTEELEMENTLIST a);


/*
Convert from a canonical string format to an RPNQuery structure.
Example input:
	pStr=cat
	pStr=or(cat[1,21],dog[1,23])
	pStr=and(cat,dog)
	pStr=or(and(cat[1,54,3,45],dog),bird)
	pStr=or(and(cat,dog),or(bird,mouse))
	
Where attribute,value pairs are in brackets.

The boolean operators "and", "or" and "andnot" are case insensitive.
Illegal characters and words in search terms are:

	[,],(,),and,or,andnot
*/
PRPNQUERY fwStrToRPN(CHAR *pOID,CHAR *pStr) {
	PRPNQUERY lRPN;
  
	lRPN=NEW(RPNQUERY);
	lRPN->rPNStructure=buildRPNStruct(pStr);
	lRPN->attributeSetId=encodeOctetStr(strdup(pOID));

	return lRPN;
}

/*
Convert from an RPNQuery structure to a canonical string format
Not done yet, obviously!
 */
CHAR *rpnToStr(PRPNQUERY pRPN) {
	return (CHAR*)"";
}

PRPNSTRUCTURE buildRPNStruct(CHAR *s) {
	PRPNSTRUCTURE r;
	int opcode;
	CHAR tmp[MAXTERMLEN];
	CHAR stmp[MAXTERMLEN];
	r=NEW(RPNSTRUCTURE);

	if((opcode=binop(s))!=-1) {
		r->which=t_binop;
		strcpy(tmp,getArg(s,opcode,1));
		r->u.binop.arg1=buildRPNStruct(tmp);
		strcpy(tmp,getArg(s,opcode,2));
		r->u.binop.arg2=buildRPNStruct(tmp);
		r->u.binop.opcode=NEW(OPERATOR);
		r->u.binop.opcode->which=opcode;
	} else {
		r->which=t_operand;
		r->u.operand=buildOperand(s);
	}

	return r; 
}

CHAR *getArg(CHAR *s, int opcode,int argNum) {
	CHAR tmp[MAXTERMLEN];
	CHAR arg[MAXTERMLEN];
	int i,j=0; 
	int numOpen=0,numClosed=0,inBrackets=0;
  
	/* strip off operator grouping */
	switch(opcode) {
		case t_and:
			for(i=4;i<strlen(s)-1;i++) 
				tmp[j++]=s[i];
			break;
		case t_or:
			for(i=3;i<strlen(s)-1;i++) 
				tmp[j++]=s[i];
			break;
		case t_andNot:
			for(i=7;i<strlen(s)-1;i++) 
				tmp[j++]=s[i];
			break;
		default:
			break;
	} 
	tmp[j]='\0';
	for(i=0;i<strlen(tmp);i++) {
		switch(tmp[i]) {
			case '(': 
				arg[i]=tmp[i];
				numOpen++;
				break;
			case ')':
				arg[i]=tmp[i];
				numClosed++;
				break;
			case '[':
				arg[i]=tmp[i];
				inBrackets=1;
				break;
			case ']':
				arg[i]=tmp[i];
				inBrackets=0;
				break;
			case ',':
				if(inBrackets==1) {
					arg[i]=tmp[i];
					break;
				}
				if(numOpen==numClosed) {
					arg[i]='\0';
					if(argNum==1)
						return arg;
					else {
						int tmpLen=strlen(arg)+1;
						for(i=0;i<strlen(tmp);i++) 
							arg[i]=
							tmp[i+tmpLen];
						arg[i+1]='\0';
						return arg;
					}
				} else arg[i]=tmp[i];
				break;
			default:
	        arg[i]=tmp[i];
		break;
		}  
	} 

	return NULL;
}

/*
This is a pretty bad function.

If string starts with:              returned is:
	"and("                             0
	"or("                              1
	"andnot("                          2
	none of the above                  -1 

The strings are case-insensitive.

*/
int binop(CHAR *s) {
	CHAR tmp[MAXTERMLEN];
	int len;

	/* begin crude, ugly hack here*/
	len=strlen(s);

	if(len>=4) {
		strncpy(tmp,s,4);
		tmp[4]='\0';
		sbUpper(tmp);
		if(!strcmp(tmp,"AND("))
			return 0;
	}
	if(len>=3) {
		strncpy(tmp,s,3);
		tmp[3]='\0';
		sbUpper(tmp);
		if(!strcmp(tmp,"OR("))
			return 1;
	}
	if(len>=7) {
		strncpy(tmp,s,7);
		tmp[7]='\0';
		sbUpper(tmp);
		if(!strcmp(tmp,"ANDNOT("))
			return 2;
	}

	return -1;
}

POPERAND buildOperand(CHAR *s) {
	POPERAND lOperand;

	lOperand=NEW(OPERAND);
	lOperand->which=t_attributesPlusTerm;
	lOperand->u.attributesPlusTerm=buildAttrPlusTerm(s);

	return lOperand;
}

PATTRIBUTESPLUSTERM buildAttrPlusTerm(CHAR *s) {
	PATTRIBUTESPLUSTERM lApt;
	CHAR term[MAXTERMLEN];
	int tmp,i,j=0,attrAvail=FALSE,cont=TRUE,state=0;
	PATTRIBUTEELEMENTLIST attrList;
	PATTRIBUTEELEMENTLIST tmpAttrList;
	CHAR tmpAttr[MAXTERMLEN];

	lApt=NEW(ATTRIBUTESPLUSTERM);
	i=(-1);
	do {   
		i++;
		if(s[i]!=BEGINATTRLISTCHAR)
			term[i]=s[i]; 
		else {
			term[i]='\0';
			attrAvail=TRUE;
		}
	} while(term[i]);

	lApt->term=encodeOctetStr(strdup(term));

	/* allocate the attribute list */
	lApt->attributeList=NEW(ATTRIBUTEELEMENTLIST);
	attrList=lApt->attributeList;
	tmpAttrList=attrList;
	attrList->item=NEW(ATTRIBUTEELEMENT);

	/* Read the attributes */
	i++; /* i now point to first attribute value */
	if(attrAvail==TRUE) {
		do {
			switch(s[i]) {
				case ',':
					tmpAttr[j]='\0';
					j=0;
					if(state==0) {
						state=1;
						attrList->item->attributeType=atol(tmpAttr);
					} else {
						state=0;
						attrList->item->attributeValue=
							atol(tmpAttr);
						attrList->next=
						NEW(ATTRIBUTEELEMENTLIST);
						attrList=attrList->next;
						attrList->item=
						NEW(ATTRIBUTEELEMENT);
					}
					break;
				case ']':
					tmpAttr[j]='\0';
					j=0;
					cont=FALSE;
					attrList->item->attributeValue=atol(tmpAttr);
					attrList->next=NULL;
					break;
				default:
					tmpAttr[j++]=s[i];
					break;
			} /* switch */
			i++;
		} while(cont);
	} /* if */

	lApt->attributeList=tmpAttrList;

	return lApt; 
}


void fwFreeRPNQuery(PRPNQUERY pQuery) {

	if(pQuery!=NULL) {
		freeRPNStructure(pQuery->rPNStructure);
		freeObjectIdentifier(pQuery->attributeSetId);
		free(pQuery);
	}
}

void fwFreeRPNStructure(PRPNSTRUCTURE r) {

	if(r!=NULL) {
		if(r->which==t_operand) {
			fwFreeRPNOperand(r->u.operand);
			free(r);
		} else {
			fwFreeRPNStructure(r->u.binop.arg1);
			fwFreeRPNStructure(r->u.binop.arg2);
			free(r->u.binop.opcode);
			free(r);
		}
	}
}

void fwFreeRPNOperand(POPERAND o) {
	if(o!=NULL) {
		if(o->which==t_attributesPlusTerm) {
			fwFreeAttributesPlusTerm(o->u.attributesPlusTerm);
			free(o);
		} else {
			freeOctetString(o->u.resultSetId);
			free(o);
		} 
	}
}

void fwFreeAttributesPlusTerm(PATTRIBUTESPLUSTERM a) {

	if(a!=NULL) {
		fwFreeAttributeList(a->attributeList);
		freeOctetString(a->term);
		free(a);
	}
}

void fwFreeAttributeList(PATTRIBUTEELEMENTLIST a) {

	while(a!=NULL) {
		fwFreeAttributeList(a->next);
		free(a->item);
		free(a);
	}
}

