/***************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery
and Retrieval, 1994.

Permission to use, copy, modify, distribute, and sell this software
and its documentation, in whole or in part, for any purpose is hereby
granted without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included
in this distribution must remain intact.
2. Users of this software agree to make their best efforts (a) to
return to MCNC any improvements or extensions that they make, so that
these may be included in future releases; and (b) to inform MCNC/CNIDR
of noteworthy uses of this software.

3. The names of MCNC and Clearinghouse for Networked Information
Discovery and Retrieval may not be used in any advertising or publicity
relating to the software without the specific, prior written permission
of MCNC/CNIDR.

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

***************************************************************/
#include "platform.h"
#ifdef PLATFORM_MSDOS
#define MAXPROFILESTR 256
#ifdef PLATFORM_BORLANDC
#include <alloc.h>
#endif
#ifdef PLATFORM_MSC
#include <malloc.h>
#endif
#endif
#include <string.h>
#include <stdio.h>
#include "netbox.h"
#include "strbox.h"
#include "berutil.h"
#include "zdefs.h"
#include "init.h"
#include "search.h"
#include "present.h"
#include "zdecode.h"
#include "zencode.h"
#include "query.h"
#include "log.h"
#include "rpn.h"
#include "zutils.h"
#include "freemem.h"
#include "search.h"

int fwSearchRequest(FILE *fp, NETBOXPROFILE*  pStream, PDU *pPDU) {
	int result;

	result=encodeZ3950PDU(pPDU,pStream);
	if(result!=1) {
		fwLog(fp,0,0,
		"search:zSearchRequest():Encode failed with %i",result);
		return -1;
	}

	return 1;
}

int fwSearchResponse(FILE *fp, NETBOXPROFILE*  pStream, PDU *pPDU) {
	int result;

	result=encodeZ3950PDU(pPDU,pStream);
	if(result!=1) {
		fwLog(fp,0,0,
		"search:zSearchResponse():Encode failed with %i",result);
		return -1;
	}

	return 1;
}

PPDU fwBuildSearchRequest(CHAR *pReferenceId, long pSmallSetUpperBound,
			long pLargeSetLowerBound, long pMedSetPresentNum,
			unsigned char pReplaceIndicator, 
			CHAR *pResultSetName, CHAR *pDatabaseNames,
			CHAR *pSmallSetElementSetNames,
			CHAR *pMedSetElementSetNames,
			CHAR *pPreferredRecordSyntax, PQUERY pQuery)
{
	PPDU lPDU;

	lPDU = (PPDU)calloc(1,sizeof(PDU));
	lPDU->which = t_searchRequest;
	lPDU->u.searchRequest=(PSEARCHREQUEST)calloc(1,sizeof(SEARCHREQUEST));

	if(*pReferenceId!=NULL)
		lPDU->u.searchRequest->referenceId=
			encodeOctetStr(strdup(pReferenceId));
	else lPDU->u.searchRequest->referenceId=NULL;

	lPDU->u.searchRequest->smallSetUpperBound=pSmallSetUpperBound;
	lPDU->u.searchRequest->largeSetLowerBound=pLargeSetLowerBound;
	lPDU->u.searchRequest->mediumSetPresentNumber=pMedSetPresentNum;
	lPDU->u.searchRequest->replaceIndicator=pReplaceIndicator;

	if(*pResultSetName!=NULL)
		lPDU->u.searchRequest->resultSetName=
			encodeOctetStr(strdup(pResultSetName));
	else
		lPDU->u.searchRequest->resultSetName=NULL;

        /* Only 1 supported */
	if(*pDatabaseNames!=NULL) {
		lPDU->u.searchRequest->databaseNames=
			(PDATABASENAMELIST)calloc(1,sizeof(DATABASENAMELIST));
		lPDU->u.searchRequest->databaseNames->item=
			encodeOctetStr(strdup(pDatabaseNames));
	} else lPDU->u.searchRequest->databaseNames= NULL;

	/* Optional and not supported yet 
	lPDU->u.searchRequest->smallSetElementSetNames=;
	lPDU->u.searchRequest->mediumSetElementSetNames=;
	*/
	if(*pPreferredRecordSyntax!=NULL)
		lPDU->u.searchRequest->preferredRecordSyntax=
			encodeOctetStr(strdup(pPreferredRecordSyntax));
        else
		lPDU->u.searchRequest->preferredRecordSyntax=NULL;
	lPDU->u.searchRequest->query = pQuery;
  
	return lPDU;
}

PPDU fwBuildSearchResponse(CHAR *pReferenceId, long pResultCount,
			long pNumRecordsReturned, long pNextResultSetPos,
			unsigned char pSearchStatus, long pResultSetStatus,
			long pPresentStatus, PRECORDS pRecords)
{
	PPDU lPDU;

	lPDU = (PPDU)calloc(1,sizeof(PDU));
	lPDU->which = t_searchResponse;
	lPDU->u.searchResponse=
		(PSEARCHRESPONSE)calloc(1,sizeof(SEARCHRESPONSE));
	lPDU->u.searchResponse->referenceId=encodeOctetStr(strdup(pReferenceId));
	lPDU->u.searchResponse->resultCount=pResultCount;
	lPDU->u.searchResponse->numberOfRecordsReturned=pNumRecordsReturned;
	lPDU->u.searchResponse->nextResultSetPosition=pNextResultSetPos;
	lPDU->u.searchResponse->searchStatus=pSearchStatus;
	lPDU->u.searchResponse->resultSetStatus=pResultSetStatus;
	lPDU->u.searchResponse->presentStatus=pPresentStatus;
	lPDU->u.searchResponse->databaseOrDiagnosticRecords=pRecords;
  
	return lPDU;
}

void fwFreeSearchRequest(PPDU pPDU) {

	freeOctetString(pPDU->u.searchRequest->referenceId);
	freeOctetString(pPDU->u.searchRequest->resultSetName);
	freeDatabaseNames(pPDU->u.searchRequest->databaseNames);
	freeElementSetNames(pPDU->u.searchRequest->smallSetElementSetNames);
	freeElementSetNames(pPDU->u.searchRequest->mediumSetElementSetNames);
	freeObjectIdentifier(pPDU->u.searchRequest->preferredRecordSyntax);
	freeQuery(pPDU->u.searchRequest->query);
	free(pPDU->u.searchRequest);
	free(pPDU);
}

void fwFreeSearchResponse(PPDU pPDU) {

	freeOctetString(pPDU->u.searchResponse->referenceId);
	freeRecords(pPDU->u.searchResponse->databaseOrDiagnosticRecords);
	free(pPDU->u.searchResponse);
	free(pPDU);
}
