/***************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery
and Retrieval, 1993.

Permission to use, copy, modify, distribute, and sell this software
and its documentation, in whole or in part, for any purpose is hereby
granted without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included
in this distribution must remain intact.
2. Users of this software agree to make their best efforts (a) to
return to MCNC any improvements or extensions that they make, so that
these may be included in future releases; and (b) to inform MCNC/CNIDR
of noteworthy uses of this software.

3. The names of MCNC and Clearinghouse for Networked Information
Discovery and Retrieval may not be used in any advertising or publicity
relating to the software without the specific, prior written permission
of MCNC/CNIDR.

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

***************************************************************/
#define PLATFORM_INC_MALLOC
#include "platform.h"
#include <stdio.h>
#include <string.h>
#include "strbox.h"
#ifdef PLATFORM_MSDOS
#include <time.h>
#else
#include <sys/time.h>
#endif
#ifndef PLATFORM_BORLANDC
#ifndef PLATFORM_SUN
CHAR* strdup(CHAR* S) {
	CHAR* T;
	   T = (CHAR*)malloc(strlen(S)+1);
        if (T != NULL) {
                strcpy(T, S);
	   }
        return T;
}
#endif
#endif

#ifdef PLATFORM_MSC
#define strdup() _strdup()
#endif

/*
FUNCTION:	sbScanTo()
PURPOSE:	Copy from src to dest up to, but not including, the
		first occurrance of a character from a	specified set
		of characters.
PRE:		dest = NULL-terminated destination string.
		src = NULL-terminated source string. 
		stopList = NULL-terminated string containing character set.
		maxLen = Maximum length of dest string.
POST:	Returns number of characters read into dest from src.
CALLS:	sbCharInSet()
*/
int sbScanTo(CHAR *dest, CHAR *src, CHAR *stopList, long maxLen) {
	int i,result;
	CHAR tempBuf[sizeof(CHAR)*2];

	for(i = 0;i < maxLen;i++) {
		if(( (result = sbCharInSet(src[i],stopList)) == 0) || (result == EOF))
			return i;
		sprintf(tempBuf,"%c",src[i]);
		strcat(dest, tempBuf);
	}
	return i;
}

/*
FUNCTION:	sbCharInSet()													      
PURPOSE:	Looks for a specified character within a specified
		set of characters.										      
PRE:		c = character for which to look.
		stopList = null-terminated string containing character set.                   
POST:	0 if c is found within stopList.
		1 if c not found within stopList.
		EOF if c or *stopList is NULL
*/
int sbCharInSet(CHAR c, CHAR *stopList) {
	int i=0;

	/* Error check */
	if(c == '\0' || stopList[0] == '\0')
	   	return EOF;
	while(stopList[i] != NULL) {
		if(c == stopList[i])
			return 0;
	      i++;
	}

   return 1;
}

/*
FUNCTION:	sbStringInSet()													      
PURPOSE:	Looks for a specified string within a specified
		set of strings.										      
PRE:		s = string for which to look.
		stopList = Array of null-terminated strings containing
		string set.                   
POST:	0 if s is found within stopList.
		1 if s not found within stopList.
*/
int sbStringInSet(CHAR *s, CHAR **stopList) {
	int i=0;

	while(*stopList[i] != NULL) {
		if(!strcmp(s,stopList[i]))
			return 0;
		i++;
	}
	return 1;
}

void sbUpper(CHAR *s) {
	int i;

	for(i=0;i<strlen(s);i++) {
		if(islower(s[i]))
			s[i]=s[i]-0x20;
        }
}

/*
FUNCTION:	theTime()
PURPOSE:	Copies the current time as a string into specified
buffer.
PRE:		buf = a pointer to an allocated char buffer.
POST:		buf = NULL-terminated string of the form:
		"Thu Feb 10 13:00:27 1994"
CALLS:		time(), sprintf(), ctime(),strlen()
*/
void theTime(CHAR *buf)
{
	time_t lTimePtr;
	int i;

	time(&lTimePtr);
	sprintf(buf,"%s",ctime(&lTimePtr));
  	/* strip off the trailing newline */
	for(i=0;i<strlen(buf);i++) {
		if(buf[i] == '\r' || buf[i] == '\n') {
			buf[i]='\0';
			return;
		}
	}
}
