/***************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery
and Retrieval, 1994.

Permission to use, copy, modify, distribute, and sell this software
and its documentation, in whole or in part, for any purpose is hereby
granted without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included
in this distribution must remain intact.
2. Users of this software agree to make their best efforts (a) to
return to MCNC any improvements or extensions that they make, so that
these may be included in future releases; and (b) to inform MCNC/CNIDR
of noteworthy uses of this software.

3. The names of MCNC and Clearinghouse for Networked Information
Discovery and Retrieval may not be used in any advertising or publicity
relating to the software without the specific, prior written permission
of MCNC/CNIDR.

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

***************************************************************/
#include <stdlib.h>
#include "libfw.h"
/*
#define _EXE_
#define FAR
#define PASCAL
*/
#include "z3950.h"

#define OK 1
#define NOTOK -1


/* Local functions */
void copyDiagMessage(char *pDest, PRECORDS pRecords);
void printPEList(PPRESENTATIONELEMENTLIST peList);
void printPE(PPRESENTATIONELEMENT tmp);
int copyRecords(PRECORDS pRecords);

/* Local defines */
#define IMPID "CNIDR"
#define IMPNAME "MS Windows Z3950 DLL (fwlib.dll)"
#define IMPVER "1.00"
#define PROTVER "11"
#define OPTIONS "11"

#define MAXBUFFERSIZE 8192
#define NETBUFFERSIZE 50000
#define MAXRECORDS 500

/* Local typedefs and structures */
typedef struct _state STATE;
typedef struct _state *PSTATE;
typedef struct _state FAR *LPSTATE;

NETBOXPROFILE gProfile;

struct _state {      
	long numHits;
	long recordCount;
	long maxRecordSize;
	RECORDS *records;
};

/* Global variables needed to maintain "state" for the DLL */
int	gLastError=OK;
int	gSocketBufSize=4096;
int	gConnection=NOTOK;	
int	gInitialized=NOTOK;
FILE 	*gLog;
STATE 	gState;
char 	*gBuffer;

#ifndef _EXE_
int FAR PASCAL LibMain (HANDLE hInstance, WORD wDataSeg, WORD cbHeapSize,
LPSTR lpCmdLine) {

	return 1;
}

#else
int main(int argc, char **argv) {
	int i,result=0;
	char *id=(char *)calloc(1,32);
	char *buf=(char *)calloc(1,4096);


	printf("trying to open connection...\n");
	if(zStartup()==NOTOK) {
		printf("failed to initialize with %i\n",zGetLastError());
		return 0;
	}

	if(zStartLogging("log.txt","w")==NOTOK) {
		printf("failed to start logging\n");
		zCleanup();
		return 0;
	}
 
	if(zOpenConnection(argv[1],atoi(argv[2]))==NOTOK) {
		printf("failed to open connection with %i\n",zGetLastError());
		zStopLogging();
		zCleanup();
		return 0;
	}

	printf("connection established...\n");
	if(zInitRequest("",id,(long)NETBUFFERSIZE,(long)NETBUFFERSIZE,"")==NOTOK) {
		printf("failed to initialize with %i\n",zGetLastError());
		zCloseConnection();
		zStopLogging();
		zCleanup();
		return 0;
	}

	if(zGetPDU(300)==NOTOK) {
		printf("failed to initialize with %i\n",zGetLastError());
		zCloseConnection();
		zStopLogging();
		zCleanup();
		return 0;
   }

	printf("initialized...\n");

	if(zSearchRequest("","Default",1,argv[3],atoi(argv[4]),argv[5])==NOTOK) {
		printf("failed to search with %i\n",zGetLastError());
		zCloseConnection();
		zStopLogging();
		zCleanup();
		return 0;
	}
	if((result=zGetPDU(300))==NOTOK) {
		printf("failed to get search pdu with %i\n",zGetLastError());
		zCloseConnection();
		zStopLogging();
		zCleanup();
		return 0;
	}

	printf("Number of hits = %ld\n",zGetSearchResults());
	if(zGetSearchResults()==-1) {
		printf("%s\n",gBuffer);
	}

	printf("received PDU of %i\n",result);
if(gState.numHits>0) {
int i;

for(i=1; i<=gState.numHits; i++){
	printf("sending present...\n");
	if(zPresentRequest("","Default",(long)i,(long)3,argv[6],0)==NOTOK) {
		printf("failed to present with %i\n",zGetLastError());
		zCloseConnection();
		zStopLogging();
		zCleanup();
		return 0;
	}

	if((result=zGetPDU(300))==NOTOK) {
		printf("failed to get present PDU with %i\n",zGetLastError());
		zCloseConnection();
		zStopLogging();
		zCleanup();
		return 0;
	}

	if(gState.recordCount>0) {
		printf("DLL has %ld records loaded\n",gState.recordCount);
		zGetRecord(0L,0L,buf,4096L);
/* 	    	printf("%s",buf);*/

                }
}
       }

	if(zCloseConnection()==NOTOK) {
		printf("failed to close connection with %i\n",zGetLastError());
		zStopLogging();
		zCleanup();
		return 0;
   }

	printf("connection closed...\n");

   zStopLogging();
	zCleanup();

	return 0;
}

#endif /* ifndef _EXE_ */

/* Start logging */
int FAR PASCAL zStartLogging(char *pFile,char *pFlag) {

	if((gLog=fopen(pFile,pFlag))==NULL) {
		gLastError=ZCANTOPENFILE;
		return NOTOK;
	}

   return OK;
}

/* Stop logging */
void FAR PASCAL zStopLogging() {

   if(gLog)
		fclose(gLog);
}

/* Return the last error code */
int FAR PASCAL zGetLastError() {

	return gLastError;
}

/* Initialize the DLL and Winsock subsystem */
int FAR PASCAL zStartup() {

	WSADATA lWsaData;
	int err;

	if(gInitialized==OK) {
		gLastError=ZINITIALIZED;
		return NOTOK;
	}

	if((err=WSAStartup(0x0101,&lWsaData))!=0) { 
		/* Failed to initialized Windows Sockets subsystem */
		gLastError=err;
		return NOTOK;
	}

	gState.numHits=0;
	gState.recordCount=0;
	gState.maxRecordSize=0;
	gState.records=NULL;

	gProfile.TimeOutSec=20;
	gProfile.TimeOutUSec=0;

	gBuffer=(char *)calloc(1,MAXBUFFERSIZE);

	gInitialized=OK;
	return OK;
}

/* Cleanup the DLL and free the Winsock subsystem */
void FAR PASCAL zCleanup() {

	if(gConnection==OK)
		netbox_Close(&gProfile);

	gInitialized=NOTOK;

	if(gState.records!=NULL)
		freeRecords(gState.records);
	free(gBuffer);

	WSACleanup();
}

/* Close the TCP/IP connection */
int FAR PASCAL zCloseConnection() {

	if(gInitialized==NOTOK) {
		gLastError=ZNOTINITIALIZED;
		return NOTOK;
	}

	gConnection=NOTOK;

	if(netbox_Close(&gProfile)!=1) {
		gLastError=WSAGetLastError();
		return NOTOK;
	}

   return OK;
}

/* Open a TCP/IP connection */
int FAR PASCAL zOpenConnection(char *pHost, int pPort) {
	int err;


	if(gInitialized==NOTOK) {
		gLastError=ZNOTINITIALIZED;
		return NOTOK;
	}

	strcpy(gProfile.HostName,pHost);
	gProfile.Port=pPort;

	if(gConnection==OK) {
		gLastError=ZNOMORECONNECTIONS;
		return NOTOK;
	}

	if(netbox_Open(&gProfile)!=1) {
		/* No error codes yet.  Rely on Winsock to give the right answer*/
		gLastError=WSAGetLastError();
		return NOTOK;
	}
	/* Need to error check on SOCKET stuff */
	gConnection=OK;
	return OK;
}

/* Send an Initialize Request PDU to the server */
int FAR PASCAL zInitRequest(char *pReferenceId, char *pIdAuthentication,
				long pPreferredMsgSize, long pMaxRecordSize,
				char *pUserInfoField) {
	PDU *lPDU;

	if(gConnection==NOTOK) {
		gLastError=ZNOTCONNECTED;
		return NOTOK;
	}

	lPDU=fwBuildInitRequest(pIdAuthentication, IMPID, IMPNAME,\
				IMPVER,	PROTVER,OPTIONS,\
				pPreferredMsgSize, pMaxRecordSize,
				pUserInfoField,pReferenceId);

	if(fwInitRequest(gLog, &gProfile, lPDU)==NOTOK) {
		gLastError=WSAGetLastError();
		fwFreeInitRequest(lPDU);
		return NOTOK;
   }
	fwLogPDU(gLog,lPDU);
	fwFreeInitRequest(lPDU);
	gState.maxRecordSize = pMaxRecordSize;

	return OK;
}

/* Wait for,fetch and decode an incoming PDU from the server */
int FAR PASCAL zGetPDU(long pTimeOut) {
	PPDU lInPDU;
	int result;
	PPRESENTATIONELEMENT presElement;
	size_t size;
	long j,i;

	if(gConnection==NOTOK) {
		gLastError=ZNOTCONNECTED;
		return NOTOK;
	}

	gProfile.TimeOutSec=pTimeOut;

	if((lInPDU=fwDecodePDU(&gProfile, gState.maxRecordSize))==NULL) {
                if(gProfile.ErrorCode==1000)
			gLastError=ZTIMEOUT;
		else
			gLastError=ZDECODEFAILED;
		return NOTOK;
	}

	fwLogPDU(gLog,lInPDU);

	switch(lInPDU->which) {
		case t_initResponse:
			result=ZINITRESPONSE;
			gState.maxRecordSize=lInPDU->u.initResponse->maximumRecordSize;
                        fwFreeInitResponse(lInPDU);
			break;
		case t_searchResponse:
			result=ZSEARCHRESPONSE;
			/* Grab anything you need from the response PDU here */
			if(lInPDU->u.searchResponse->resultCount==0 &&
				lInPDU->u.searchResponse->searchStatus==0) {
				/* Make bad assumption that a nonsurrogateDiag was returned */
				copyDiagMessage(gBuffer,
					lInPDU->u.searchResponse->databaseOrDiagnosticRecords);
					gState.numHits=-1L; /* Tell caller that a diag returned */
			} else
				gState.numHits=lInPDU->u.searchResponse->resultCount;
			fwFreeSearchResponse(lInPDU);
			break;
		case t_presentResponse: {
			PPRESENTRESPONSE PR;
			PR=lInPDU->u.presentResponse;
			result=ZPRESENTRESPONSE;
                        /* Free any existing records */
			if(gState.records!=NULL) {
				gState.recordCount=0;
				freeRecords(gState.records);
			}
                        /* Did you get any records back? */
			if(PR->numberOfRecordsReturned>0) {
                        	/* Was it only a diagnostic? */
				if(isDiagnostic(PR->databaseOrDiagnosticRecords)==1)
					copyDiagMessage(gBuffer,PR->databaseOrDiagnosticRecords);
				else {
                                	/* Got real records */
					gState.records=\
						PR->databaseOrDiagnosticRecords;
					gState.recordCount=numRecords(gState.records);
				}
			}

			/* Since we are relying on presentResponse->records,
			 * we need to manually free the PDU, leaving
			 * records intact.
                         */
			freeOctetString(lInPDU->u.presentResponse->referenceId);
			free(lInPDU->u.presentResponse);
			free(lInPDU);
			break;
		}
/*
		case t_accessControlRequest:
			result=ZACCESSREQUEST;
			break;
*/
		default:
			gLastError=ZUNSUPPORTEDPDU;
	}

	return result;
}

/* Send a Search Request to the server */
int FAR PASCAL zSearchRequest(char *pReferenceId, char *resultSetName,
		int pReplaceIndicator, char *databaseNames,
		int attributeSetId, char *query) {
	PRPNQUERY lRPNQuery;
	PQUERY lQuery;
	unsigned char lReplaceIndicator;
	PPDU lPDU;
	char *lOID;

	if(gConnection==NOTOK) {
		gLastError=ZNOTCONNECTED;
		return NOTOK;
	}
	/* Clear appropriate portions of state table */
	gState.numHits=0;

	switch(attributeSetId) {
		case BIB1_ATTR_SET:
			lOID = strdup(BIB1);
			break;
		case STAS_ATTR_SET:
			lOID = strdup(STAS);
			break;
		default:
			lOID = strdup(BIB1);
	}

	lRPNQuery=fwStrToRPN(lOID,query);
	lQuery=fwBuildRPNQuery(lRPNQuery);

	if(pReplaceIndicator==0)
		lReplaceIndicator=(unsigned char)'0';
	else
		lReplaceIndicator=(unsigned char)'1';

	lPDU=fwBuildSearchRequest(pReferenceId,
					0,
					2,
					1,
					lReplaceIndicator,
					resultSetName,
					databaseNames,
	       				"",
					"",
					"",
					lQuery);

	 fwLogPDU(gLog,lPDU);
	 if(fwSearchRequest(gLog, &gProfile, lPDU)==NOTOK) {
		gLastError=WSAGetLastError();
		if(gLastError==0) {
			/* Error wasn't due to a Winsock problem, so call it
			 * undocumented for now
			 */
			 gLastError=ZUNDOCUMENTED;
      }
		fwFreeSearchRequest(lPDU);
		return NOTOK;
	}
	fwFreeSearchRequest(lPDU);

   return OK;
}

int FAR PASCAL zPresentRequest(char *pReferenceId, char *pResultSetId,
			long pResultSetStartPoint, long pNumRecsRequested,
			char *pElementSetNames,int pPreferredRecordSyntax) {
	PPDU lPDU;
	char *lOID;
	char *lReferenceId;
	PELEMENTSETNAMES lEsn;

	if(gConnection==NOTOK) {
		gLastError=ZNOTCONNECTED;
		return NOTOK;
	}

	/* check record syntax */
	switch(pPreferredRecordSyntax) {
		case 0:
			lOID=strdup(USMARC);
			break;
		case 1:
			lOID=strdup(SUTRS);
			break;
		default:
			lOID=NULL;
	}

	lEsn=(PELEMENTSETNAMES)calloc(1,sizeof(ELEMENTSETNAMES));
	if(pElementSetNames!=NULL) {
		lEsn->which=t_generic;
		lEsn->u.generic=encodeOctetStr(strdup(pElementSetNames));

   } else {
		lEsn->which=t_generic;
		lEsn->u.generic=encodeOctetStr(strdup("B"));
	}


	lPDU=fwBuildPresentRequest(pReferenceId, pResultSetId,
		pResultSetStartPoint, pNumRecsRequested,lEsn,
		lOID);

	 fwLogPDU(gLog,lPDU);
	 if(fwPresentRequest(gLog, &gProfile, lPDU)==NOTOK) {
		gLastError=WSAGetLastError();
		if(gLastError==0) {
			/* Error wasn't due to a Winsock problem, so call it
			 * undocumented for now
			 */
			 gLastError=ZUNDOCUMENTED;
      }
		fwFreePresentRequest(lPDU);
		return NOTOK;
	}
	fwFreePresentRequest(lPDU);

	return OK;
}
long FAR PASCAL zGetSearchResults() {

	return gState.numHits;
}

long FAR PASCAL zGetPresentResults() {

	return gState.recordCount;
}

long FAR PASCAL zGetBuffer(char *pBuffer, long pMaxLen) {
	long len;

	if(strlen(gBuffer) > pMaxLen)
		len=pMaxLen;
	else
		len=strlen(gBuffer);

        
	strncpy(pBuffer, gBuffer, len);

	return len;
}

void copyDiagMessage(char *pDest, PRECORDS pRecords) {

	if(pRecords!=NULL) {
		switch(pRecords->which) {
			case t_dataBaseOrSurDiagnostics:
				break;
			case t_nonSurrogateDiagnostic:
				sprintf(pDest,"%ld:%s",
					pRecords->u.nonSurrogateDiagnostic->condition,
					decodeOctetStr(pRecords->u.nonSurrogateDiagnostic->addinfo));
				break;
			default:
				sprintf(pDest,"Diagnostic Record Type Unknown: %i",
					pRecords->which);
		}
   }
}

long FAR PASCAL zGetRecord(long pRecNum, long pStartPoint,char *pBuf, \
			long pMaxBufSize)
{
	char *p;
	long len,recType;

	if(pRecNum > gState.recordCount)
		return -1;

	len=copyRecord(gState.records,pRecNum,pStartPoint,pBuf,pMaxBufSize);

	return len;
}
