/***************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery
and Retrieval, 1994.

Permission to use, copy, modify, distribute, and sell this software
and its documentation, in whole or in part, for any purpose is hereby
granted without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included
in this distribution must remain intact.
2. Users of this software agree to make their best efforts (a) to
return to MCNC any improvements or extensions that they make, so that
these may be included in future releases; and (b) to inform MCNC/CNIDR
of noteworthy uses of this software.

3. The names of MCNC and Clearinghouse for Networked Information
Discovery and Retrieval may not be used in any advertising or publicity
relating to the software without the specific, prior written permission
of MCNC/CNIDR.

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

***************************************************************/
#include "platform.h"
#ifdef PLATFORM_MSDOS
#ifdef PLATFORM_BORLANDC
#include <alloc.h>
#endif
#ifdef PLATFORM_MSC
#include <malloc.h>
#endif
#endif
#include <stdio.h>
#include "netbox.h"
#include "berutil.h"
#include "ztags.h"
#include "zdefs.h"
#include "zinit.h"
#include "zsearch.h"
#include "zpresent.h"
#include "zdecode.h"
#include "imem.h"
#include "zaccess.h"
#define NEW(a) (a*)calloc(1,sizeof(a));

PDU *fwDecodePDU(NETBOXPROFILE* profile, long maxBufferSize)
{
	CHAR FAR *b;
	DATA_DIR *d;
	PPDU p;
	PINITIALIZEREQUEST InitReq;
	PINITIALIZERESPONSE InitResp;
	PSEARCHREQUEST SearchReq;
	PSEARCHRESPONSE SearchResp;
	PPRESENTREQUEST PresentReq;
   	PPRESENTRESPONSE PresentResp;
	long result;

	b=(CHAR FAR *)imem_calloc(1,maxBufferSize);

	if(b==NULL)
		return(NULL);
	result=zGetBER(profile,b,maxBufferSize);
	if(result<=0) {
		/* didn't read anything for some reason */
		/* probably timed out */
		/* need some error checking here */
		return NULL;
	}

      d=dalloc(zDefaultNodeCount);
	if(!bld_dir(b,d))
		return(NULL);
	p=NEW(PDU);
	if(p==NULL)
		return(NULL);

	switch(d->fldid){
		case zInitRequestTag:
			/* decode an init request - server function */
			p->which=t_initRequest;
			p->u.initRequest=decodeInitRequest(d);
			break;
		case zInitResponseTag:
			/* decode init response */
			p->which=t_initResponse;
			p->u.initResponse=decodeInitResponse(d);	
			break;
		case zSearchRequestTag:
			/* decode search request */
			p->which=t_searchRequest;
			p->u.searchRequest=decodeSearchRequest(d);
			break;
		case zSearchResponseTag:
			/* decode search response */
			p->which=t_searchResponse;
			p->u.searchResponse=decodeSearchResponse(d);
			break;
		case zPresentRequestTag:
			/* decode present request */
			break;
		case zPresentResponseTag:
			/* decode present response */
			p->which=t_presentResponse;
			p->u.presentResponse=decodePresentResponse(d);
			break;
		case zAccessControlRequestTag:
			p->which=t_accessControlRequest;
			p->u.accessControlRequest=decodeAccessControlRequest(d);
                        break;
		case zAccessControlResponseTag:
			p->which=t_accessControlResponse;
			p->u.accessControlResponse=decodeAccessControlResponse(d);
			break;
	}
	dfree(d);

 	return(p);
}

long zGetBER(NETBOXPROFILE *profile, CHAR * FAR b, long maxbuffersize)
{
	long len,remainder,tlen;

	if(netbox_WaitForData(profile)==0)
		return -1;

	len=netbox_GetBuffer(profile,b,6);
	if(len)
		while(!IsCompleteBER(b,len,&remainder))
		{
			if(netbox_WaitForData(profile)==0)
				return -1;
			tlen=netbox_GetBuffer(profile,b+len,remainder?remainder:1);
			if(!tlen)
				return(-1);
			len+=tlen;
		}
        return(len);
}
