/***************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery
and Retrieval, 1994.

Permission to use, copy, modify, distribute, and sell this software
and its documentation, in whole or in part, for any purpose is hereby
granted without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included
in this distribution must remain intact.
2. Users of this software agree to make their best efforts (a) to
return to MCNC any improvements or extensions that they make, so that
these may be included in future releases; and (b) to inform MCNC/CNIDR
of noteworthy uses of this software.

3. The names of MCNC and Clearinghouse for Networked Information
Discovery and Retrieval may not be used in any advertising or publicity
relating to the software without the specific, prior written permission
of MCNC/CNIDR.

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

***************************************************************/
#ifndef _zdefs_
#define _zdefs_

#ifndef PLATFORM_MSDOS
#define FAR
#endif

/* Include lots of typedefs for the following structures */
#include "ztypes.h"

/* attribute sets */
#define BIB1 "1.2.840.10003.3.1"
#define STAS "1.2.840.10003.3.1000.6.1"

/* record syntaxes */
#define USMARC "1.2.840.10003.5.10"
#define SUTRS "1.2.840.10003.5.101"

/* BER/ASN-1/Z39.50 specific data structures */
struct _octetString {
	unsigned char  *data;
	long length;
};

struct _presentationElementList {
	PPRESENTATIONELEMENT item;
	PPRESENTATIONELEMENTLIST next;
};

struct _presentationElement {
    long    PETag;
    int     PEForm;
    union {
	PPRESENTATIONELEMENTLIST list;
	POCTETSTRING data;
    } u;
};

struct _records {
	enum {
		t_dataBaseOrSurDiagnostics, t_nonSurrogateDiagnostic
	} which;
	union {
		struct _namePlusRecordList {
			PNAMEPLUSRECORD item;
			PNAMEPLUSRECORDLIST next;
		} *dataBaseOrSurDiagnostics;
		PDIAGRECORD nonSurrogateDiagnostic;
	} u;
};

struct _namePlusRecord {
	POCTETSTRING databaseName;
	struct _dbRecordOrSurrogate {
		enum {
			t_databaseRecord, t_surrogateDiagnostic
		} which;
		union {
			PEXTERNAL databaseRecord;
			PDIAGRECORD surrogateDiagnostic;
		} u;
	} recordOrSurrogate;
};

struct _diagRecord {
	POBJECTIDENTIFIER diagnosticSetId;
	long condition;
	POCTETSTRING addinfo;
};

struct _elementSetNames {
	enum {
		t_generic, t_databaseSpecific
	} which;
	union {
		POCTETSTRING generic;
		struct _elementSetPlusDB {
			struct {
				POCTETSTRING databaseName;
				POCTETSTRING elementSetName;
			}       item;
			PELEMENTSETPLUSDB next;
		} *databaseSpecific;
	} u;
};

struct _attributesPlusTerm {
	PATTRIBUTEELEMENTLIST attributeList;
	POCTETSTRING term;
};

struct _attributeElementList {
	PATTRIBUTEELEMENT item;
	PATTRIBUTEELEMENTLIST next;
};

struct _attributeElement {
	long attributeType;
	long attributeValue;
};

struct _query {
	enum {
		t_type_0, t_type_1, t_type_2, t_type_100, t_type_101
	} which;
	union {
		PANY type_0;
		PRPNQUERY type_1;
		POCTETSTRING type_2;
		POCTETSTRING type_100;
		POCTETSTRING type_101;
	} u;
};

struct _rPNQuery {
	POBJECTIDENTIFIER attributeSetId;
	PRPNSTRUCTURE rPNStructure;
};

struct _rPNStructure {
	enum {
		t_operand, t_binop
	} which;
	union {
		POPERAND operand;
		struct {
			PRPNSTRUCTURE arg1;
			PRPNSTRUCTURE arg2;
			POPERATOR opcode;
		} binop;
	} u;
};

struct _operand {
	enum {
		t_attributesPlusTerm, t_resultSetId
	} which;
	union {
		PATTRIBUTESPLUSTERM attributesPlusTerm;
		POCTETSTRING resultSetId;
	} u;
};

struct _operator {
	enum {
		t_and, t_or, t_andNot
	} which;
	union {
		unsigned char and;
		unsigned char or;
		unsigned char andNot;
	} u;
};

struct _pdu {
	enum {
		t_initRequest, t_initResponse,
		t_searchRequest, t_searchResponse,
		t_presentRequest, t_presentResponse,
		t_accessControlRequest, t_accessControlResponse,
                t_deleteResultSetRequest, t_deleteResultSetResponse
	} which;
	union {
		PINITIALIZEREQUEST initRequest;
		PINITIALIZERESPONSE initResponse;
		PSEARCHREQUEST searchRequest;
		PSEARCHRESPONSE searchResponse;
		PPRESENTREQUEST presentRequest;
		PPRESENTRESPONSE presentResponse;
		PACCESSCONTROLREQUEST 	accessControlRequest;
		PACCESSCONTROLRESPONSE accessControlResponse;
		PDELETERESULTSETREQUEST deleteResultSetRequest;
		PDELETERESULTSETRESPONSE deleteResultSetResponse;
	} u;
};

struct _initializeRequest {
	POCTETSTRING referenceId;
	POCTETSTRING protocolVersion;
	POCTETSTRING options;
	long preferredMessageSize;
	long maximumRecordSize;
	PANY idAuthentication;
	POCTETSTRING implementationName;
	POCTETSTRING implementationVersion;
	POCTETSTRING implementationId;
	PEXTERNAL userInformationField;
};

struct _initializeResponse {
	POCTETSTRING referenceId;
	PBITSTRING protocolVersion;
	PBITSTRING options;
	long preferredMessageSize;
	long maximumRecordSize;
	unsigned char result;
	POCTETSTRING implementationId;
	POCTETSTRING implementationName;
	POCTETSTRING implementationVersion;
	PEXTERNAL userInformationField;
};

struct _searchRequest {
	POCTETSTRING referenceId;
	long    smallSetUpperBound;
	long    largeSetLowerBound;
	long    mediumSetPresentNumber;
	unsigned char   replaceIndicator;
	POCTETSTRING resultSetName;
	struct _databaseNameList {
		POCTETSTRING item;
		PDATABASENAMELIST next;
	} *databaseNames;
	PELEMENTSETNAMES smallSetElementSetNames;
	PELEMENTSETNAMES mediumSetElementSetNames;
	POBJECTIDENTIFIER preferredRecordSyntax;
	PQUERY query;
};

struct _searchResponse {
	POCTETSTRING referenceId;
	long resultCount;
	long numberOfRecordsReturned;
	long nextResultSetPosition;
	unsigned char searchStatus;
	long resultSetStatus;
	long presentStatus;
	PRECORDS databaseOrDiagnosticRecords;
};

struct _presentRequest {
	POCTETSTRING referenceId;
	POCTETSTRING resultSetId;
	long resultSetStartPoint;
	long numberOfRecordsRequested;
	PELEMENTSETNAMES elementSetNames;
	POBJECTIDENTIFIER preferredRecordSyntax;
};

struct _presentResponse {
	POCTETSTRING referenceId;
	long numberOfRecordsReturned;
	long nextResultSetPosition;
	long presentStatus;
	PRECORDS databaseOrDiagnosticRecords;
};

struct _accessControlRequest{
	POCTETSTRING referenceId;
	enum
		{t_rqoldWay,t_rqnewWay
	}which;
	 union{
		POCTETSTRING oldWay;
		PEXTERNAL newWay;
	}u;
};
struct _accessControlResponse{
       POCTETSTRING referenceId;
	enum
		{t_rpoldWay,t_rpnewWay
	}which;
	 union{
		POCTETSTRING oldWay;
		PEXTERNAL newWay;
	}u;
};

struct _deleteResultSetRequest {
	POCTETSTRING referenceId;
	struct _deleteOperation {
		long operation;
		enum {
			t_list, t_all
		} which;
	} deleteOperation;
	struct _resultSetList {
		POCTETSTRING item;
		PRESULTSETLIST next;
	}*resultSetList;
};

struct _deleteResultSetResponse {
	POCTETSTRING referenceId;
	long deleteOperationStatus;
	PLISTSTATUSES deleteListStatuses;
	long numberNotDeleted;
	PLISTSTATUSES bulkStatuses;
	POCTETSTRING deleteMessage;
};

struct _listStatuses {
	struct {
		POCTETSTRING resultSetId;
		long deleteSetStatus;
	} item;
	PLISTSTATUSES next;
};

struct _waisrec{
        POCTETSTRING headline;
        POCTETSTRING personalName;
        POCTETSTRING date;
        long score;
        POCTETSTRING docId;
        PWAISRECLIST reclist;
};

struct _waisreclist{
        POCTETSTRING item;
        PWAISRECLIST next;
};
#endif /* _zdefs_ */
