/***************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery
and Retrieval, 1994.

Permission to use, copy, modify, distribute, and sell this software
and its documentation, in whole or in part, for any purpose is hereby
granted without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included
in this distribution must remain intact.
2. Users of this software agree to make their best efforts (a) to
return to MCNC any improvements or extensions that they make, so that
these may be included in future releases; and (b) to inform MCNC/CNIDR
of noteworthy uses of this software.

3. The names of MCNC and Clearinghouse for Networked Information
Discovery and Retrieval may not be used in any advertising or publicity
relating to the software without the specific, prior written permission
of MCNC/CNIDR.

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

***************************************************************/
#include "platform.h"
#ifdef PLATFORM_MSDOS
#ifdef PLATFORM_BORLANDC
#include <alloc.h>
#endif
#ifdef PLATFORM_MSC
#include <malloc.h>
#endif
#endif
#include <string.h>
#include <stdio.h>
#include "netbox.h"
#include "berutil.h"
#include "ztags.h"
#include "zdefs.h"
#include "zutils.h"
#include "zinit.h"
#define NEW(a) (a*)calloc(1,sizeof(a));

CHAR *encodeInitializeRequest(PINITIALIZEREQUEST Init, long *len)
{
	CHAR *outputRecord, *temp;
	DATA_DIR *d,*subdir;

	d=dmake(zInitRequestTag,ASN1_CONTEXT,zDefaultNodeCount);
        if(Init->referenceId!=NULL)
		daddchar(d,zReferenceIdTag,ASN1_CONTEXT,
			(CHAR *)Init->referenceId->data,
			Init->referenceId->length);
	daddbits(d,zProtocolVersionTag,ASN1_CONTEXT,(char *)decodeOctetStr(Init->protocolVersion));
	daddbits(d,zOptionsTag,ASN1_CONTEXT,(char *)decodeOctetStr(Init->options));
	daddnum(d,zPreferredMessageSizeTag,ASN1_CONTEXT,
		(CHAR *)&Init->preferredMessageSize,sizeof(Init->preferredMessageSize));
	daddnum(d,zMaximumRecordSizeTag,ASN1_CONTEXT,(CHAR *)&Init->maximumRecordSize,
		sizeof(Init->maximumRecordSize));
	if(Init->idAuthentication!=NULL) {
		subdir=daddtag(d,Init->idAuthentication->PETag,ASN1_CONTEXT);
		subdir=daddtag(subdir,ASN1_EXTERNAL,ASN1_UNIVERSAL);
		daddchar(subdir,1,ASN1_CONTEXT,
			(CHAR *)Init->idAuthentication->u.data->data,
			Init->idAuthentication->u.data->length);
	}
	if(Init->implementationId!=NULL)
		daddchar(d,zImplementationIdTag,ASN1_CONTEXT,
			(CHAR *)Init->implementationId->data,
			Init->implementationId->length);
	if(Init->implementationName!=NULL)
		daddchar(d,zImplementationNameTag,ASN1_CONTEXT,
			(CHAR *)Init->implementationName->data,
			Init->implementationName->length);
	if(Init->implementationVersion!=NULL)
		daddchar(d,zImplementationVersionTag,ASN1_CONTEXT,
			(CHAR *)Init->implementationVersion->data,
			Init->implementationVersion->length);
	temp=bld_rec(d,len);
	outputRecord=(CHAR *)calloc(1,*len);
	COPY(outputRecord,temp,*len);
	dfree(d);

	return(outputRecord);
}

PINITIALIZERESPONSE decodeInitResponse(DATA_DIR *d)
{
	PINITIALIZERESPONSE IR;
	DATA_DIR *sub;

	IR=NEW(INITIALIZERESPONSE);
	IR->referenceId=NULL;
	IR->protocolVersion=NULL;
	IR->options=NULL;
	IR->implementationId=NULL;
	IR->implementationName=NULL;
	IR->implementationVersion=NULL;
	IR->userInformationField=NULL;

	for(sub=d->ptr.child; sub; sub=sub->next){
		switch(sub->fldid){
			case zReferenceIdTag: {
				CHAR *tbuf;
				tbuf=(CHAR *)calloc(1,sub->count+1);
				COPY(tbuf,sub->ptr.data,sub->count);
				IR->referenceId=encodeOctetStr(tbuf);
				break;
			}
			case zProtocolVersionTag:{
				IR->protocolVersion=
				encodeBitStr(sub->ptr.data,sub->count);
				break;
			}
			case zOptionsTag:{
				IR->options=encodeBitStr(sub->ptr.data,
					sub->count);
				break;
			}
			case zPreferredMessageSizeTag:{
				long pms;
				pms=dgetnum(sub);
				IR->preferredMessageSize=pms;
				break;
			}
			case zMaximumRecordSizeTag:{
				long mrs;
				mrs=dgetnum(sub);
				IR->maximumRecordSize=mrs;
				break;
			}
			case zResultTag:   {
				CHAR data;
				data=(unsigned char)(*(unsigned char *)sub->ptr.data);
				IR->result=data;
				break;
			}
			case zImplementationIdTag:{
				CHAR *tbuf;
				tbuf=(CHAR *)calloc(1,sub->count+1);
				COPY(tbuf,sub->ptr.data,sub->count);
				IR->implementationId=encodeOctetStr(tbuf);
				break;
			}
			case zImplementationNameTag:{
				CHAR *tbuf;
				tbuf=(CHAR *)calloc(1,sub->count+1);
				COPY(tbuf,sub->ptr.data,sub->count);
				IR->implementationName=encodeOctetStr(tbuf);
				break;
			}
			case zImplementationVersionTag:{
				CHAR *tbuf;
				tbuf=(CHAR *)calloc(1,sub->count+1);
				COPY(tbuf,sub->ptr.data,sub->count);
				IR->implementationVersion=encodeOctetStr(tbuf);
				break;
			}
		}
	}

	return IR;
}

PINITIALIZEREQUEST decodeInitRequest(DATA_DIR *d)
{
	PINITIALIZEREQUEST IR;
	DATA_DIR *sub;

	IR=NEW(INITIALIZEREQUEST);
	IR->referenceId=NULL;
	IR->protocolVersion=NULL;
	IR->options=NULL;
	IR->implementationId=NULL;
	IR->implementationName=NULL;
	IR->implementationVersion=NULL;
	IR->userInformationField=NULL;

	for(sub=d->ptr.child; sub; sub=sub->next){
		switch(sub->fldid){
			case zReferenceIdTag: {
				CHAR *tbuf;
				tbuf=(CHAR *)calloc(1,sub->count+1);
				COPY(tbuf,sub->ptr.data,sub->count);
				IR->referenceId=encodeOctetStr(tbuf);
				break;
			}
			case zProtocolVersionTag:{
				IR->protocolVersion=
				encodeBitStr(sub->ptr.data,sub->count);
				break;
			}
			case zOptionsTag:{
				IR->options=encodeBitStr(sub->ptr.data,
					sub->count);
				break;
			}
			case zPreferredMessageSizeTag:{
				long pms;
				pms=dgetnum(sub);
				IR->preferredMessageSize=pms;
				break;
			}
			case zMaximumRecordSizeTag:{
				long mrs;
				mrs=dgetnum(sub);
				IR->maximumRecordSize=mrs;
				break;
			}
			case zImplementationIdTag:{
				CHAR *tbuf;
				tbuf=(CHAR *)calloc(1,sub->count+1);
				COPY(tbuf,sub->ptr.data,sub->count);
				IR->implementationId=encodeOctetStr(tbuf);
				break;
			}
			case zImplementationNameTag:{
				CHAR *tbuf;
				tbuf=(CHAR *)calloc(1,sub->count+1);
				COPY(tbuf,sub->ptr.data,sub->count);
				IR->implementationName=encodeOctetStr(tbuf);
				break;
			}
			case zImplementationVersionTag:{
				CHAR *tbuf;
				tbuf=(CHAR *)calloc(1,sub->count+1);
				COPY(tbuf,sub->ptr.data,sub->count);
				IR->implementationVersion=encodeOctetStr(tbuf);
				break;
			}
		
		}
	}
        return(IR);

}
CHAR *encodeInitializeResponse(PINITIALIZERESPONSE Init, long *len)
{
	CHAR *outputRecord, *temp;
	DATA_DIR *d,*subdir;

	d=dmake(zInitResponseTag,ASN1_CONTEXT,zDefaultNodeCount);
        if(Init->referenceId!=NULL)
		daddchar(d,zReferenceIdTag,ASN1_CONTEXT,
			(CHAR *)Init->referenceId->data,
			Init->referenceId->length);
	daddbits(d,zProtocolVersionTag,ASN1_CONTEXT,(char *)decodeOctetStr(Init->protocolVersion));
	daddbits(d,zOptionsTag,ASN1_CONTEXT,(char *)decodeOctetStr(Init->options));
	daddnum(d,zPreferredMessageSizeTag,ASN1_CONTEXT,
		(CHAR *)&Init->preferredMessageSize,sizeof(Init->preferredMessageSize));
	daddnum(d,zMaximumRecordSizeTag,ASN1_CONTEXT,(CHAR *)&Init->maximumRecordSize,
		sizeof(Init->maximumRecordSize));
	daddchar(d,zResultTag,ASN1_CONTEXT,(CHAR *)&Init->result,sizeof(Init->result));


	if(Init->implementationId!=NULL)
		daddchar(d,zImplementationIdTag,ASN1_CONTEXT,
			(CHAR *)Init->implementationId->data,
			Init->implementationId->length);
	if(Init->implementationName!=NULL)
		daddchar(d,zImplementationNameTag,ASN1_CONTEXT,
			(CHAR *)Init->implementationName->data,
			Init->implementationName->length);
	if(Init->implementationVersion!=NULL)
		daddchar(d,zImplementationVersionTag,ASN1_CONTEXT,
			(CHAR *)Init->implementationVersion->data,
			Init->implementationVersion->length);
	temp=bld_rec(d,len);
	outputRecord=(CHAR *)calloc(1,*len);
	COPY(outputRecord,temp,*len);
	dfree(d);

	return(outputRecord);

}
