/***************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery
and Retrieval, 1994.

Permission to use, copy, modify, distribute, and sell this software
and its documentation, in whole or in part, for any purpose is hereby
granted without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included
in this distribution must remain intact.
2. Users of this software agree to make their best efforts (a) to
return to MCNC any improvements or extensions that they make, so that
these may be included in future releases; and (b) to inform MCNC/CNIDR
of noteworthy uses of this software.

3. The names of MCNC and Clearinghouse for Networked Information
Discovery and Retrieval may not be used in any advertising or publicity
relating to the software without the specific, prior written permission
of MCNC/CNIDR.

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

***************************************************************/
#include "platform.h"
#ifdef PLATFORM_MSDOS
#ifdef PLATFORM_BORLANDC
#include <alloc.h>
#endif
#ifdef PLATFORM_MSC
#include <malloc.h>
#endif
#endif
#include <stdio.h>
#include <string.h>
#include "netbox.h"
#include "berutil.h"
#include "zdefs.h"
#include "strbox.h"
#include "zutils.h"
#include "ztags.h"
#include "ztypes.h"
#include "zrecords.h"
#define NEW(a) (a*)calloc(1,sizeof(a));

PDIAGRECORD decodeDiagRec(DATA_DIR *d)
{
	DATA_DIR *sub;
	PDIAGRECORD DR;

	DR=NEW(DIAGRECORD);
	DR->diagnosticSetId=NULL;
	DR->addinfo=NULL;

	for(sub=d->ptr.child; sub; sub=sub->next){
		switch(sub->fldid){
			case zOIDTag:
				DR->diagnosticSetId=
					encodeOctetStr(strdup((CHAR*)dgetoid(sub)));
				break;
			case zIntegerTag:
				DR->condition=dgetnum(sub);
				break;
			case zVisibleStringTag: {
				CHAR *buf;
				buf=(CHAR *)calloc(1,sub->count+1);
				COPY(buf,sub->ptr.data,sub->count);
				DR->addinfo=encodeOctetStr(strdup(buf));
				break;
			}
			default:
                        	/* protocol error */
				return NULL;
		}
	}

	return(DR);
}

PRECORDS decodeRecords(DATA_DIR *d,long *errTag)
{
 	DATA_DIR *sub2;
	PRECORDS Rec;
	int i;
	PNAMEPLUSRECORDLIST place;
	PNAMEPLUSRECORDLIST place2;

	Rec=NEW(RECORDS);

	switch(d->fldid) {
		case zDatabaseOrSurDiagnosticsTag: {
			Rec->which=t_dataBaseOrSurDiagnostics;
			place=NEW(NAMEPLUSRECORDLIST);
			place2=place;
			sub2=d->ptr.child;
			for(i=0; i<d->count; i++,sub2=sub2->next){
				if(sub2->fldid!=ASN1_SEQUENCE){
					*errTag=zProtocolErrorTag;
					return(NULL);
				}
				place->item=decodeNPR(sub2,errTag);
				if(i==d->count-1)
                                	place->next=NULL;
				else {
					place->next=NEW(NAMEPLUSRECORDLIST);
					place=place->next;
                        	}
			}
			Rec->u.dataBaseOrSurDiagnostics=place2;
			break;
		}
		case zNonSurrogateDiagnosticTag: {
			Rec->which=t_nonSurrogateDiagnostic;
			Rec->u.nonSurrogateDiagnostic=decodeDiagRec(d);
			break;
		}
		default:
			return NULL;
	}
	*errTag=zProtocolOk;

	return(Rec);
 }

PPRESENTATIONELEMENT decodePE(DATA_DIR *d){
	PPRESENTATIONELEMENT pe;
	pe=NEW(PRESENTATIONELEMENT);
	
	if(d->form > 0) {
		/* Constructed presentation element */
                int i;
		PPRESENTATIONELEMENTLIST peList;
		DATA_DIR *child;
		peList=NEW(PRESENTATIONELEMENTLIST);
		pe->PETag=d->fldid;
		pe->PEForm=d->form;
		pe->u.list=peList;
		child=d->ptr.child;
		for(i=0; i < d->count; i++,child=child->next){
			peList->item=decodePE(child);
			if(i < (d->count-1)) {
				peList->next=NEW(PRESENTATIONELEMENTLIST);			                	
				peList=peList->next;
			} else
				peList->next=NULL;
		}
	} else {
		/* Not a constructed element */
		pe->PETag=d->fldid;
		pe->PEForm=d->form;
		pe->u.data=encodeNOctetStr(d->ptr.data,d->count);
	}

	return(pe);
}

PNAMEPLUSRECORD decodeNPR(DATA_DIR *d, long *errTag) {
	PNAMEPLUSRECORD npr;
	DATA_DIR *sub4;
        DATA_DIR *child1;
	int i;

	npr=(PNAMEPLUSRECORD)calloc(1,sizeof(NAMEPLUSRECORD));
	child1=d->ptr.child;

	for(i=0;i < d->count;i++,child1=child1->next) {
		switch(child1->fldid) {
			case 0: {
				/* database name */
				npr->databaseName=(POCTETSTRING)calloc(1,sizeof(OCTETSTRING));
				npr->databaseName->length=child1->count;
				npr->databaseName=encodeNOctetStr(child1->ptr.data, child1->count);
				break;
			}
			case 1: {
		         	/* record or surrogate diagnostic */
				sub4=child1->ptr.child;
				switch(sub4->fldid){
					case 1:{ /* database Record */
						/* it's external */
						DATA_DIR *sub5;
						sub5=sub4->ptr.child;
						npr->recordOrSurrogate.which=t_databaseRecord;
						npr->recordOrSurrogate.u.databaseRecord=decodePE(sub5);
						break;
			                }
					case 2:
						/* surrogate diagnostic */
						npr->recordOrSurrogate.which=t_surrogateDiagnostic;
						npr->recordOrSurrogate.u.surrogateDiagnostic=decodeDiagRec(sub4);
						break;
					default:
						/* unknown record type */
						return NULL;
				} /* switch sub4->fldid */
				break;
			} /* case 1: */
			default:
                        	return NULL;
		} /* switch */
	} /* for */

	return npr;
}

