/***************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery
and Retrieval, 1994.

Permission to use, copy, modify, distribute, and sell this software
and its documentation, in whole or in part, for any purpose is hereby
granted without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included
in this distribution must remain intact.
2. Users of this software agree to make their best efforts (a) to
return to MCNC any improvements or extensions that they make, so that
these may be included in future releases; and (b) to inform MCNC/CNIDR
of noteworthy uses of this software.

3. The names of MCNC and Clearinghouse for Networked Information
Discovery and Retrieval may not be used in any advertising or publicity
relating to the software without the specific, prior written permission
of MCNC/CNIDR.

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

***************************************************************/
#define PLATFORM_INC_MALLOC
#include "platform.h"
#include <stdio.h>
#include <string.h>
#include "berutil.h"
#include "netbox.h"
#include "ztypes.h"
#include "zdefs.h"
#include "ztags.h"
#include "zutils.h"
#include "zresult.h"

#define NEW(a) (a*)calloc(1,sizeof(a))

PDELETERESULTSETRESPONSE decodeDeleteResultSetResponse(DATA_DIR *d)
{
	PDELETERESULTSETRESPONSE DR;
	DATA_DIR *sub;
	int i;

	if((DR=NEW(DELETERESULTSETRESPONSE))==NULL)
		return NULL;
	DR->referenceId=NULL;
	sub = d->ptr.child;

	for(i=0; i<d->count; i++,sub=sub->next){
		switch(sub->fldid){
		 	case zReferenceIdTag: {
				char *tbuf;

				tbuf=(char *)calloc(1,sub->count+1) ;
				COPY(tbuf,sub->ptr.data,sub->count);
				DR->referenceId=encodeOctetStr(tbuf);
				break;
			}
			case zDeleteOperationStatusTag:
			{
				char *tbuf;
                                DATA_DIR *sub1;
				sub1=sub->ptr.child;
				if(sub1->fldid == zDeleteSetStatusTag)
					DR->deleteOperationStatus=
						dgetnum(sub1);
				else
					/* protocol error */
                                        return NULL;
				break;
			}
			case zDeleteListStatusesTag:
				DR->deleteListStatuses=
					decodeListStatuses(sub);
                        	break;
			case zNumberNotDeletedTag:
				DR->numberNotDeleted=dgetnum(sub);
                        	break;
			case zBulkStatusesTag:
				DR->bulkStatuses=
					decodeListStatuses(sub);
				break;
			case zDeleteMessageTag: {
				char *tbuf;
				tbuf=(char *)calloc(1,sub->count+1);
				COPY(tbuf,sub->ptr.data,sub->count);
				DR->deleteMessage=encodeOctetStr(tbuf);
				break;
			}
		} /* switch */
	} /* for */
	return(DR);
}

PLISTSTATUSES decodeListStatuses(DATA_DIR *d) {
	PLISTSTATUSES l;
	DATA_DIR *sub1, *sub2;
	int i;
 
	if((l=NEW(LISTSTATUSES))==NULL)
		return(NULL);
	/* expecting a sequence tag */
	if(d->fldid != ASN1_SEQUENCE)
		/* protocol error */
		return NULL;
        sub1=d->ptr.child;

	/* expecting one or more triples of
	   sequence, resultsetid,deletesetstatus
	*/
	for(i=0;i<sub1->count;i++,sub1=sub1->next) {
		if(sub1->fldid != ASN1_SEQUENCE)
			/* protocol error */
			return NULL;
                sub2=sub1->ptr.child;
		switch(sub2->fldid) {
			case zResultSetIdTag: {
				char *tbuf;
				tbuf=(char *)calloc(1,sub2->count+1);
				COPY(tbuf,sub2->ptr.data,sub2->count);
				l->item.resultSetId=encodeOctetStr(tbuf);
				break;
			}
			case zDeleteSetStatusTag:
				l->item.deleteSetStatus=dgetnum(sub2);
				break;
			default:
				/* protocol error */
                                return NULL;
		} /* switch */
		if(i!=sub1->count-1) {
			l->next=NEW(LISTSTATUSES);
			l=l->next;
		} else
			l->next = NULL;                	
	} /* for */

	return(l);
}

char *encodeDeleteResultSetRequest(PDELETERESULTSETREQUEST DR, long *len)
{
	char *outputRecord, *temp;
	DATA_DIR *d, *deleteOpNode;
	char buf[256];

	d=dmake(zDeleteResultSetRequestTag,ASN1_CONTEXT,zDefaultNodeCount);
	if(DR->referenceId!=NULL){
		daddchar(d,zReferenceIdTag,ASN1_CONTEXT,
			(unsigned char *)DR->referenceId->data,
			DR->referenceId->length);
	}
	deleteOpNode=daddtag(d,zDeleteOperationTag,ASN1_CONTEXT);
	if(DR->deleteOperation.which == t_list) {
		daddnum(deleteOpNode,zDeleteOperationListTag,ASN1_CONTEXT,
		(char *)&DR->deleteOperation.operation,
		sizeof(DR->deleteOperation.operation));
	} else {
		daddnum(deleteOpNode,zDeleteOperationAllTag,ASN1_CONTEXT,
		(char *)&DR->deleteOperation.operation,
		sizeof(DR->deleteOperation.operation));
	}
	if(DR->resultSetList!=NULL){
		PRESULTSETLIST list;
		list=DR->resultSetList;
		while(list!=NULL) {
			DATA_DIR *seqNode;
			seqNode=daddtag(d, ASN1_SEQUENCE, ASN1_UNIVERSAL);
			daddchar(d,zResultSetIdTag,ASN1_CONTEXT,
				(unsigned char *)list->item->data,
				list->item->length);
			list = list->next;
                }
        }
	temp=bld_rec(d,len);
	outputRecord=(char *)calloc(1,*len);
	COPY(outputRecord,temp,*len);
	dfree(d);

	return(outputRecord);
}


