/***************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery
and Retrieval, 1994.

Permission to use, copy, modify, distribute, and sell this software
and its documentation, in whole or in part, for any purpose is hereby
granted without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included
in this distribution must remain intact.
2. Users of this software agree to make their best efforts (a) to
return to MCNC any improvements or extensions that they make, so that
these may be included in future releases; and (b) to inform MCNC/CNIDR
of noteworthy uses of this software.

3. The names of MCNC and Clearinghouse for Networked Information
Discovery and Retrieval may not be used in any advertising or publicity
relating to the software without the specific, prior written permission
of MCNC/CNIDR.

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

***************************************************************/
#include "platform.h"
#ifdef PLATFORM_MSDOS
#ifdef PLATFORM_BORLANDC
#include <alloc.h>
#endif
#ifdef PLATFORM_MSC
#include <malloc.h>
#endif
#endif
#include <stdio.h>
#include <string.h>
#include "berutil.h"
#include "strbox.h"
#include "netbox.h"
#include "zdefs.h"
#include "ztags.h"
#include "zrecords.h"
#include "zutils.h"
#include "zsearch.h"
#define NEW(a) (a*)calloc(1,sizeof(a));

/*
FUNCTION:	encodeSearchRequest()
PURPOSE:	Builds an OCLC BER record from a SearchRequest structure.
PRE:		SR = pointer to a valid, completed SearchRequest structure.
		len = long pointer in which the length of the resulting
		is returned.
POST:		NULL on error.
		Pointer to encoded BER record.
		len = length of the record.
NOTES:		When building the SearchRequest structure, make sure you set
		all pointers not used to NULL.  Also, NULL the end of
		any lists below the SearchRequest structure.
*/
CHAR *encodeSearchRequest(PSEARCHREQUEST SR, long *len) {
	DATA_DIR *mainNode;
	DATA_DIR *dbNamesNode;
	DATA_DIR *queryNode;
	DATA_DIR *smallSetNode;
	DATA_DIR *medSetNode;
	DATA_DIR *elementSetNamesNode;
	CHAR *outputRecord, *temp;
	PDATABASENAMELIST dbNamesPtr=SR->databaseNames;

	mainNode = dmake(zSearchRequestTag, ASN1_CONTEXT, zDefaultNodeCount);

        if(SR->referenceId!=NULL)
		daddchar(mainNode, zReferenceIdTag, ASN1_CONTEXT, 
			(CHAR *)SR->referenceId->data, SR->referenceId->length);

	daddnum(mainNode, zSmallSetUpperBoundTag, ASN1_CONTEXT, 
		(CHAR *)&SR->smallSetUpperBound,sizeof(SR->smallSetUpperBound));

	daddnum(mainNode, zLargeSetLowerBoundTag, ASN1_CONTEXT,
		(CHAR *)&SR->largeSetLowerBound,sizeof(SR->largeSetLowerBound));

	daddnum(mainNode, zMediumSetPresentNumberTag, ASN1_CONTEXT, 
		(CHAR *)&SR->mediumSetPresentNumber,
		sizeof(SR->mediumSetPresentNumber));

	daddchar(mainNode, zReplaceIndicatorTag, ASN1_CONTEXT,
		(CHAR *)&SR->replaceIndicator, sizeof(SR->replaceIndicator));

	if(SR->resultSetName!=NULL)
		daddchar(mainNode, zResultSetNameTag, ASN1_CONTEXT,
			(CHAR *)SR->resultSetName->data, SR->resultSetName->length);

	if(SR->databaseNames!=NULL) {
		dbNamesNode=daddtag(mainNode, zDatabaseNamesTag, ASN1_CONTEXT);
		while(dbNamesPtr!=NULL) {
			daddchar(dbNamesNode, zDatabaseNameTag, ASN1_CONTEXT,
				(CHAR *)dbNamesPtr->item->data,
				dbNamesPtr->item->length);
		     	dbNamesPtr=dbNamesPtr->next;
		}
	}

	if(SR->smallSetElementSetNames!=NULL) {
		smallSetNode=daddtag(mainNode, zSmallSetElementSetNamesTag,
			ASN1_CONTEXT);
		elementSetNamesNode=daddtag(smallSetNode,
			zElementSetNamesTag, ASN1_CONTEXT);
		if(SR->smallSetElementSetNames->which==t_generic) {
			/* generic */
			daddchar(elementSetNamesNode, zGenericTag, ASN1_CONTEXT,
				SR->smallSetElementSetNames->u.generic->data,
				SR->smallSetElementSetNames->u.generic->length);
		} else {
			/* databaseSpecific */
			DATA_DIR *dbSpecificNode;
			PELEMENTSETPLUSDB esnPtr;
			esnPtr=SR->smallSetElementSetNames->u.databaseSpecific;
			dbSpecificNode=daddtag(elementSetNamesNode,
				zDatabaseSpecificTag, ASN1_CONTEXT);
			while(esnPtr) {
				daddchar(dbSpecificNode, zDatabaseNameTag,
					ASN1_CONTEXT,
					esnPtr->item.databaseName->data,
					esnPtr->item.databaseName->length);
				daddchar(dbSpecificNode, zElementSetNameTag,
					ASN1_CONTEXT,
					esnPtr->item.elementSetName->data,
					esnPtr->item.elementSetName->length);
				esnPtr=esnPtr->next;
               		}
		}
	}

	if(SR->mediumSetElementSetNames!=NULL) {
		medSetNode=daddtag(mainNode, zMediumSetElementSetNamesTag,
			ASN1_CONTEXT);
		elementSetNamesNode=daddtag(medSetNode, zElementSetNamesTag,
			ASN1_CONTEXT);
		if(SR->mediumSetElementSetNames->which==t_generic) {
			/* generic */
			daddchar(elementSetNamesNode, zGenericTag, ASN1_CONTEXT,
				SR->mediumSetElementSetNames->u.generic->data,
				SR->mediumSetElementSetNames->u.generic->length);
		} else {
			/* databaseSpecific */
			DATA_DIR *dbSpecificNode;
			PELEMENTSETPLUSDB esnPtr;
			esnPtr=SR->mediumSetElementSetNames->u.databaseSpecific;
			dbSpecificNode=daddtag(elementSetNamesNode,
				zDatabaseSpecificTag, ASN1_CONTEXT);
			while(esnPtr) {
				daddchar(dbSpecificNode, zDatabaseNameTag,
					ASN1_CONTEXT,
					esnPtr->item.databaseName->data,
					esnPtr->item.databaseName->length);
				daddchar(dbSpecificNode, zElementSetNameTag,
					ASN1_CONTEXT,
					esnPtr->item.elementSetName->data,
					esnPtr->item.elementSetName->length);
				esnPtr=esnPtr->next;
			}
		}
	}

	/* Query (Only type1 supported) */
	queryNode=daddtag(mainNode, zQueryTag, ASN1_CONTEXT);
	switch(SR->query->which) {
		case t_type_1: {
			DATA_DIR *rpnQueryNode;
			CHAR *bf=(CHAR *)calloc(1,64);

			PRPNQUERY rpn=SR->query->u.type_1;
			rpnQueryNode=daddtag(queryNode, zType1QueryTag, ASN1_CONTEXT);

			strncpy(bf,rpn->attributeSetId->data,rpn->attributeSetId->length);
			daddoid(rpnQueryNode, ASN1_OBJECTIDENTIFIER,
				ASN1_UNIVERSAL,(char*)bf);


			dAddRPNStructure(rpnQueryNode,rpn->rPNStructure);
			break;
		}
		default:
		 	/* Unsupported Query Type */
			return NULL;
	}
	temp=bld_rec(mainNode,len);

	outputRecord=(CHAR *)calloc(1,*len);
	COPY(outputRecord,temp,*len);
	
	dfree(mainNode);

	return(outputRecord);
}


/*
FUNCTION:	dAddRPNStructure()
PURPOSE:	Builds an OCLC BER sub-tree from an RPNSTRUCTURE structure.
PRE:		rpnStruct = pointer to a valid, completed RPNSTRUCTURE.
		parent = BER tree pointer to which to attach this subtree.
POST:		NULL on error.
		Pointer to BER subtree.
NOTES:		When building the SearchRequest structure, make sure you set
		all pointers not used to NULL.  Also, NULL the end of
		any lists below the SearchRequest structure.
*/
DATA_DIR *dAddRPNStructure(DATA_DIR *parent,PRPNSTRUCTURE rpnStruct) {
	DATA_DIR *rpnsNode;

	switch(rpnStruct->which) {
		case t_operand: {
			/* Single Operand */
			POPERAND operand=rpnStruct->u.operand;

			/* Operand */
			rpnsNode=daddtag(parent,zOperandTag,ASN1_CONTEXT);

			if(operand->which==t_attributesPlusTerm) {
				/* AttributesPlusTerm */
				if(dAddAttrPlusTerm(rpnsNode,
					operand->u.attributesPlusTerm)==NULL)
				return NULL;
			} else {
				/* ResultSetId */
				daddchar(rpnsNode, zResultSetIdTag,
				ASN1_CONTEXT,
				rpnStruct->u.operand->u.resultSetId->data,
				rpnStruct->u.operand->u.resultSetId->length);
			}
			break;
		}
		case t_binop: {
			/* Boolean Query */
			DATA_DIR *rpnsOperatorNode;

			/* Boolean Query */
			rpnsNode=daddtag(parent,zBooleanQueryTag,ASN1_CONTEXT);

			/* Arg1 */
			if(dAddRPNStructure(rpnsNode,
				rpnStruct->u.binop.arg1)==NULL)
				return NULL;			

			/* Arg2 */
			if(dAddRPNStructure(rpnsNode,
				rpnStruct->u.binop.arg2)==NULL)
				return NULL;			
			/* Opcode */
			rpnsOperatorNode=daddtag(rpnsNode,zOperatorTag,
				ASN1_CONTEXT);
			switch(rpnStruct->u.binop.opcode->which) {
				case t_and:
					daddchar(rpnsOperatorNode,
						zAndTag,ASN1_CONTEXT,(CHAR*)"",0L);
					break;
				case t_or:
					daddchar(rpnsOperatorNode,
						zOrTag,ASN1_CONTEXT,(CHAR*)"",0L);
					break;
				case t_andNot:
					daddchar(rpnsOperatorNode,
						zAndNotTag,ASN1_CONTEXT,(CHAR*)"",0L);
					break;
				default:
					/* shouldn't happen, but...*/
					daddchar(rpnsOperatorNode,zOrTag,
						ASN1_CONTEXT,(CHAR*)"",0L);
					break;
			}
		}
		default:
			return NULL;
	}	

	return rpnsNode;
}

/*
FUNCTION:	dAddRPNStructure()
PURPOSE:	Builds an OCLC BER sub-tree from an RPNStructure structure.
PRE:		rpnStruct = pointer to a valid, completed RPNStructure structure.
		parent = BER tree pointer to which to attach this subtree.
POST:		NULL on error.
		Pointer to BER subtree.
NOTES:		When building the SearchRequest structure, make sure you set
		all pointers not used to NULL.  Also, NULL the end of
		any lists below the SearchRequest structure.
*/
DATA_DIR *dAddAttrPlusTerm(DATA_DIR *parent,PATTRIBUTESPLUSTERM attrTerm) {
	DATA_DIR *aptNode;
	DATA_DIR *attrListNode;
	PATTRIBUTEELEMENTLIST attrList=attrTerm->attributeList;


	aptNode=daddtag(parent, zAttributesPlusTermTag, ASN1_CONTEXT);

	/* Add Attributes */
	attrListNode=daddtag(aptNode,zAttributeListTag,ASN1_CONTEXT);

	while(attrList!=NULL) {
		DATA_DIR *attrListSequenceNode;
		attrListSequenceNode=daddtag(attrListNode,ASN1_SEQUENCE,
			ASN1_UNIVERSAL);
		daddnum(attrListSequenceNode, zAttributeTypeTag,ASN1_CONTEXT,
			(CHAR *)&attrList->item->attributeType, 
			sizeof(attrList->item->attributeType));
		daddnum(attrListSequenceNode, zAttributeValueTag,ASN1_CONTEXT,
			(CHAR *)&attrList->item->attributeValue,
			sizeof(attrList->item->attributeValue));
		attrList=attrList->next;
	}

	/* Add Term */
	daddchar(aptNode, zQueryTermTag, ASN1_CONTEXT,
		(CHAR *)attrTerm->term->data,attrTerm->term->length);

	return aptNode;
}

/*
FUNCTION:	decodeSearchResponse()
PURPOSE:	Builds SearchResponse structure from an OCLC BER record.
PRE:		SR = pointer to a valid, completed SearchRequest structure.
		len = long pointer in which the length of the resulting
		is returned.
POST:		NULL on error.
		Pointer to encoded BER record.
		len = length of the record.
NOTES:		When building the SearchRequest structure, make sure you set
		all pointers not used to NULL.  Also, NULL the end of
		any lists below the SearchRequest structure.
*/
PSEARCHRESPONSE decodeSearchResponse(DATA_DIR *d) {
	PSEARCHRESPONSE SR;
	DATA_DIR *sub;
	long errTag;

	SR=(PSEARCHRESPONSE)malloc(sizeof(SEARCHRESPONSE));
	SR->referenceId=NULL;
	SR->databaseOrDiagnosticRecords=NULL;

	for(sub=d->ptr.child; sub; sub=sub->next){
		switch(sub->fldid){
			case zReferenceIdTag: {
				CHAR *tbuf;
				tbuf=(CHAR *)calloc(1,sub->count+1);
				COPY(tbuf,sub->ptr.data,sub->count);
				SR->referenceId=encodeOctetStr(tbuf);
				break;
			}
			case zResultCountTag:{
				long i;
				i=dgetnum(sub);
				SR->resultCount=i;
				break;
			}
			case zNumberOfRecordsReturnedTag:{
				long i;
				i=dgetnum(sub);
				SR->numberOfRecordsReturned=i;
				break;
			}
			case zNextResultSetPositionTag:{
				long i;
				i=dgetnum(sub);
				SR->nextResultSetPosition=i;
				break;
			}
			case zSearchStatusTag:{
				long i;
				i=dgetnum(sub);
				SR->searchStatus=i;
				break;
			}
			case zResultSetStatusTag:{
				long i;
				i=dgetnum(sub);
				SR->resultSetStatus=i;
				break;
			}
			case zPresentStatusTag:{
				long i;
				i=dgetnum(sub);
				SR->presentStatus=i;
				break;
			}
			case zDatabaseOrSurDiagnosticsTag: {
				SR->databaseOrDiagnosticRecords=decodeRecords(sub,&errTag);
                                break;
			}
			case zNonSurrogateDiagnosticTag: {
				SR->databaseOrDiagnosticRecords=decodeRecords(sub,&errTag);
                                break;
			}
			default:
				/* protocol error */
                                return NULL;
		}
	}

	return SR;
}
PSEARCHREQUEST decodeSearchRequest(DATA_DIR *d) {
        PSEARCHREQUEST SR;
        DATA_DIR *sub;
        long errTag;

        SR=(PSEARCHREQUEST)malloc(sizeof(SEARCHREQUEST));
        SR->referenceId=NULL;

        for(sub=d->ptr.child; sub; sub=sub->next){
                switch(sub->fldid){
                        case zReferenceIdTag: {
				CHAR *tbuf;
				tbuf=(CHAR *)calloc(1,sub->count+1);
                                COPY(tbuf,sub->ptr.data,sub->count);
                                SR->referenceId=encodeOctetStr(tbuf);
                                break;
                        }
			case zSmallSetUpperBoundTag:
				SR->smallSetUpperBound=dgetnum(sub);
				break;
			case zLargeSetLowerBoundTag:
				SR->largeSetLowerBound=dgetnum(sub);
				break;
			case zMediumSetPresentNumberTag:
				SR->mediumSetPresentNumber=dgetnum(sub);
				break;
			case zReplaceIndicatorTag:
				SR->replaceIndicator=*(sub->ptr.data);
				break;
			case zResultSetNameTag:
				{
				CHAR *bf=(CHAR *)calloc(1,sub->count+1);
				strncpy(bf,sub->ptr.data,sub->count);
				SR->resultSetName=encodeOctetStr(bf);
				break;
				}
			case zDatabaseNamesTag:
				{
				DATA_DIR *sub;
				PDATABASENAMELIST ptr, 
					dbNamesPtr= NEW(DATABASENAMELIST);
				ptr=dbNamesPtr;
				for(sub=d->ptr.child; sub; sub=sub->next){
					CHAR *bf=(CHAR *)calloc(1,sub->count+1);
					COPY(bf,sub->ptr.data,sub->count);
					dbNamesPtr->item=encodeOctetStr(bf);
					dbNamesPtr->next=NEW(DATABASENAMELIST);
					dbNamesPtr=dbNamesPtr->next;
				}
	
				free(dbNamesPtr);
				SR->databaseNames=ptr;
				break;
			}
			case zPreferredRecordSyntaxTag:
				SR->preferredRecordSyntax=encodeOctetStr(strdup((CHAR*)dgetoid(d)));
				break;
			case zQueryTag:
				/* we told you it's not a server yet! */
				SR->query=decodeQuery(d);
				break;
		}
	}
	return(SR);
}

PQUERY decodeQuery(DATA_DIR *d)
{
	DATA_DIR *qLevel, *rpnLevel;

	qLevel=d->ptr.child;
	return(NULL);
}

