/***************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery
and Retrieval, 1994.

Permission to use, copy, modify, distribute, and sell this software
and its documentation, in whole or in part, for any purpose is hereby
granted without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included
in this distribution must remain intact.
2. Users of this software agree to make their best efforts (a) to
return to MCNC any improvements or extensions that they make, so that
these may be included in future releases; and (b) to inform MCNC/CNIDR
of noteworthy uses of this software.

3. The names of MCNC and Clearinghouse for Networked Information
Discovery and Retrieval may not be used in any advertising or publicity
relating to the software without the specific, prior written permission
of MCNC/CNIDR.

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

***************************************************************/
#include "platform.h"
#ifdef PLATFORM_MSDOS
#ifdef PLATFORM_BORLANDC
#include <alloc.h>
#endif
#ifdef PLATFORM_MSC
#include <malloc.h>
#endif
#endif
#include <string.h>
#include <stdio.h>
#include "strbox.h"
#include "berutil.h"
#include "netbox.h"
#include "zdefs.h"
#include "zutils.h"

/*
FUNCTION:	encodeOctetStr()
PURPOSE:	Copies the specified NULL-terminated char buf into an
		octetString structure.
PRE:		s = a character buffer.
POST:		On success, returns an octetString structure.
		On failure, returns NULL.
CALLS:		calloc(), strlen()
*/
POCTETSTRING encodeOctetStr(CHAR *s)
{
	POCTETSTRING t;

	if((t=(POCTETSTRING)calloc(1,sizeof(OCTETSTRING)))==NULL)
        	return NULL;
	t->length=strlen(s);
	t->data=s;

	return(t);
}

/*
FUNCTION:	encodeNOctetStr()
PURPOSE:	Copies the specified char buf of specified length into an
		octetString structure.
PRE:		s = a character buffer.
POST:		On success, returns an octetString structure.
		On failure, returns NULL.
CALLS:		calloc(), strlen()
*/
POCTETSTRING encodeNOctetStr(CHAR *s,long len)
{
	POCTETSTRING t;

	if((t=(POCTETSTRING)malloc(sizeof(OCTETSTRING)))==NULL)
        	return NULL;
	t->length=len;
	t->data=(CHAR *)calloc(1,len+2L);
	COPY(t->data,s,len);

	return(t);
}

/*
FUNCTION:	decodeOctetStr()
PURPOSE:	Copies the specified octetString structure into a NULL-
		terminated string buffer.
PRE:		s = an octet string structure.
POST:		On success, returns a NULL-terminated string.
		On failure, returns NULL.
CALLS:		COPY(), calloc()
*/
CHAR *decodeOctetStr(POCTETSTRING s)
{
	CHAR *p;

	if((p=(CHAR *)calloc(1,s->length+1))==NULL)
        	return NULL;
	COPY(p,s->data,s->length);

	return(p);
}

/*
FUNCTION:	encodeBitStr()
PURPOSE:	Copies the specified char buf of specified length into a
		bitString structure.
PRE:		s = a character buffer.
		len = length of s.
POST:		On success, returns a bitString structure.
		On failure, returns NULL.
CALLS:		COPY(), calloc()
*/
PBITSTRING encodeBitStr(CHAR *s, long len)
{
	PBITSTRING t;

	t=(PBITSTRING)calloc(1,sizeof(BITSTRING));
	t->length=len;
	t->data=(CHAR *)calloc(1,len+1);
	COPY(t->data,s,len);

	return(t);
}

/*
FUNCTION:	decodeBitStr()
PURPOSE:	Copies the specified bitString structure into a NULL-
		terminated string buffer.
PRE:		s = a bit string structure.
POST:		On success, returns a NULL-terminated string.
		On failure, returns NULL.
CALLS:		COPY(), calloc()
*/
CHAR *decodeBitStr(PBITSTRING b)
{
	CHAR *p;

	if((p=(CHAR *)calloc(1,b->length+1))==NULL)
        	return NULL;
	COPY(p,b->data,b->length);

	return(p);
}

