/*************************************************************************

Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery and
Retrieval, 1994. 

Permission to use, copy, modify, distribute, and sell this software and
its documentation, in whole or in part, for any purpose is hereby granted
without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included in
this distribution must remain intact. 

2. Users of this software agree to make their best efforts (a) to return
to MCNC any improvements or extensions that they make, so that these may
be included in future releases; and (b) to inform MCNC/CNIDR of noteworthy
uses of this software. 

3. The names of MCNC and Clearinghouse for Networked Information Discovery
and Retrieval may not be used in any advertising or publicity relating to
the software without the specific, prior written permission of MCNC/CNIDR. 

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR
IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 

*************************************************************************/
/*
	Usage: fwais [-i[filename][,group]] [-ooption[=setting]] [...]
*/
#define PLATFORM_INC_MALLOC
#include "libfw.h"
#include "ini.h"
#include "fwais.h"

/* Private functions */
int GetPrivateProfileInt(CHAR *pszSection, char *pszEntry, int pszDefault, 
	CHAR *pszFileName);
long GetPrivateProfileLong(CHAR *pszSection, char *pszEntry, long pszDefault, 
	CHAR *pszFileName);
void DisplayRecords(SESSION *Session, SESSIONSTUFF *stuff);
int GetCmdLineSettings(SESSION* S, int ac, CHAR** argv);
int ReadProfileSettings(SESSION *S, CHAR *File, CHAR *Group);
int Present(SESSION *session,SESSIONSTUFF *stuff,NETBOXPROFILE *p);
int Search(PSESSION session,SESSIONSTUFF *stuff, NETBOXPROFILE *Profile);
void StoreRecords(SESSION *Session, SESSIONSTUFF *Stuff, PDU *P);
void DisplayError(SESSION *Session, SESSIONSTUFF *stuff);


/* 
Hard-coded defaults for Session parameters.
These can be individually overridden, first by a .ini file specified at
the command line, followed by individual command line Session parameter 
arguments.
*/
SESSION DEFAULTSESSION = {
	"",			/* Host */
        210,			/* Port */
	30L,			/* TimeOut */
	"",			/* Output File */
	"",			/* Stats File */
	"fwais.log",		/* Log File */
	"",			/* IdAuthentication */
	"23",			/* ImpId */
	"CNIDR FWAIS CLIENT/GATEWAY", /* ImpName */
	"1.0",			/* ImpVersion */
	"11",			/* ProtocolVersion */
	"11",			/* Options */
	30000L,			/* PreferredMessageSize */
	30000L,			/* MaxRecSize */
	"",			/* UserInformationField */
	"",			/* ReferenceId */
	"",			/* ReferenceId */
	0l,			/* SmallSetUpperBound */	
	2l,			/* LargeSetLowerBound */
	1l,			/* MediumSetPresentNum */
	'1',			/* ReplaceIndicator */
	"Default",		/* ResultSetNames */
	"CAT",			/* DBNames */
	"",			/* SmallSetElementSetNames */
	"",			/* MediumSetElementSetNames */
	"",			/* PreferredRecordSyntax */
	"",			/* Query */
	"1.2.840.10003.3.1",	/* AttributeSet */
	"",			/* ReferenceId */
	"Default",		/* ResultSEtId */
	1l,			/* StartPoint */
	1l,			/* NumRecsReq */
	"B",			/* ElementSetNames */	
	"1.2.840.10003.5.10" 	/* PreferredRecordSyntax */
};

int main(int argc, CHAR **argv) {
	OCTETSTRING *tempOctetString;
	SESSIONSTUFF stuff;
	SESSION Session;
	CHAR Host[64];
	CHAR DatabaseName[64];
	int Port;
	PDU *RequestPDU;
	PDU *ResponsePDU;
	QUERY *Z3950Query;
	RPNQUERY *RPNQuery;
	NETBOXPROFILE Profile;
	int Result,i;
	CHAR *tempBuf;
	
	Session=DEFAULTSESSION;	
	ReadProfileSettings(&Session,(CHAR*)PROFILEFN,(CHAR*)PROFILEGROUP); 
	if((Result=GetCmdLineSettings(&Session, argc, argv))!=0)
		printf("Failed to read command line option #%i\n",Result);

	strcpy(Profile.HostName,Session.Host);
	Profile.Port=Session.Port;
	Profile.TimeOutSec=Session.TimeOut;

	if(strlen(Profile.HostName)==0) {
		printf("Bad host name\n");
		return 0;
	}
	for(i=0;i<MAXNUMRECS;i++) {	
		stuff.records[i]=(OCTETSTRING *)calloc(1,sizeof(OCTETSTRING));
	}

	stuff.numHits=0;
	stuff.numRecords=0;
	
	/* Open the log file */
	stuff.logFp=fopen(Session.LogFile,"w");

	/* Connect to the remote server */	
	if(netbox_Open(&Profile) != 1) {
		sprintf(stuff.Diag,"%s:%s,%i",FWNOCONNECT,Session.Host,
			Session.Port);
		DisplayError(&Session,&stuff);
		return 0;
	}

	if(Init(&Session, &stuff, &Profile)==(-1)) {
		netbox_Close(&Profile);
		return 0;
	}
	
	if((Result=GetPDU(&Session, &stuff, &Profile))==(-1)) {
		if(Result!=t_initResponse) {
			printf("Didn't get PDU I wanted\n");
			netbox_Close(&Profile);
			return 0;
		}	
	}

	if(Search(&Session, &stuff, &Profile)==(-1)) {
		printf("Search failed\n");
		netbox_Close(&Profile);
		return 0;
	}

	if((Result=GetPDU(&Session, &stuff, &Profile))==(-1)) {
		if(Result!=t_searchResponse) {
			printf("Didn't get PDU I wanted\n");
			netbox_Close(&Profile);
			return 0;
		}	
		return 0;
	}

	/*printf("Number of hits was %ld\n",stuff.numHits);*/

	if(stuff.numRecords > 0)
		DisplayRecords(&Session, &stuff);	
	
	if(Present(&Session, &stuff, &Profile)==(-1)) {
		printf("Present failed\n");
		netbox_Close(&Profile);
		return 0;
	}

	if(stuff.numRecords > 0)
		DisplayRecords(&Session, &stuff);	
	
	for(i=0;i < MAXNUMRECS;i++) {
		freeOctetString(stuff.records[i]);
	}
	
	/*printf("Done.\n");*/
	netbox_Close(&Profile);
	return 0;
}

/* Totally dependant on session parameters. */
int GetPDU(PSESSION Session,SESSIONSTUFF *stuff, NETBOXPROFILE *Profile) {
	PDU *R;
	int Result;

	/* Decode an incoming PDU */
	if((R=fwDecodePDU(Profile, MAXNETBUFFERSIZE))==NULL) {
		sprintf(stuff->Diag,"%s:%s",FWNOGETRES,Session->Host);
		DisplayError(Session, stuff);
		return(-1);
	}
	
	fwLogPDU(stuff->logFp, R);

	switch(R->which) {
		case t_initResponse: {
			INITIALIZERESPONSE *I;

			I=R->u.initResponse;
			Result=t_initResponse;
			/* Grab the max and preferred record sizes*/
			Session->iMaxRecSize=I->maximumRecordSize;
			Session->iPreferredMsgSize=I->preferredMessageSize;
			if(I->result == 0) {
				sprintf(stuff->Diag,"%s:%s",FWINITDENIED,
					Session->Host);
				DisplayError(Session,stuff);
				Result=(-1);	
			}
			fwFreeInitResponse(R);
			break;
		}
		case t_searchResponse: {
			SEARCHRESPONSE *S;

			S=R->u.searchResponse;
			stuff->numHits=S->resultCount;	
			if(stuff->numHits==0) {
				sprintf(stuff->Diag,"%s:%s",FWNOHITS,
					Session->sQuery);
				DisplayError(Session,stuff);
				Result=(-1);	
			} else {
				if(S->numberOfRecordsReturned > 0)
					StoreRecords(Session,stuff,R);
				if(stuff->Diag[0]!=NULL) {
					DisplayError(Session,stuff);
					Result=(-1);
				} else
					Result=t_searchResponse;
			}
			fwFreeSearchResponse(R);
			break;
		}
		case t_presentResponse: {
			/* Present is a special case */
			/* see Present() */
			Result=t_presentResponse;
			fwFreePresentResponse(R);
			break;
		}
		default:
			Result=(-1);
	}

	return(Result);
}

void StoreRecords(SESSION *Session, SESSIONSTUFF *Stuff, PDU *P) {
	PRECORDS R;
	long NumRecs,i,len,temp;
	CHAR *Buf;

	/* Clear appropriate stuff in stuff */
	Stuff->Diag[0]='\0';
	
	if(P->which == t_searchResponse) {
		R=P->u.searchResponse->databaseOrDiagnosticRecords;
		NumRecs=P->u.searchResponse->numberOfRecordsReturned;		
	} else {
		R=P->u.presentResponse->databaseOrDiagnosticRecords;
		NumRecs=P->u.presentResponse->numberOfRecordsReturned;		
	}

	if(NumRecs > 0) {
		if(numRecords(R) > 0) {
			/*There were records returned on the response*/
			if(isDiagnostic(R) == 1) {
				/* Its a nonSurrogateDiagnostic */
				strcpy(Stuff->Diag,
					decodeOctetStr(
					R->u.nonSurrogateDiagnostic->addinfo));
				DisplayError(Session,Stuff);
			} else {
				/* There are _real_ records */
				Buf=(CHAR *)calloc(1,Session->iMaxRecSize);
				temp=numRecords(R)+Stuff->numRecords;	
				for(i=Stuff->numRecords;i<temp;i++) {
					len=copyRecord(R,i-Stuff->numRecords,
						0L,(char*)Buf, Session->iMaxRecSize);
					if(len>0)
						Stuff->records[i]=
							encodeNOctetStr(Buf,
							len);
					else {
						Stuff->records[i]='\0';
					}
				}
				Stuff->numRecords=temp;
				free(Buf);	
			}
                        freeRecords(R);
                }
	} 
}

/* Totally dependant on session parameters. */
int Init(PSESSION Session,SESSIONSTUFF *stuff, NETBOXPROFILE *Profile) {
	PDU *RequestPDU;
	
	/* Build an Init Request PDU and send it */
	RequestPDU=fwBuildInitRequest(Session->iAuthentication,
		Session->iImpID,
		Session->iImpName,
		Session->iImpVersion,
		Session->iProtocolVersion,
		Session->iOptions,
		Session->iPreferredMsgSize,
		Session->iMaxRecSize,
		Session->iUserInformationField,
		Session->iReferenceId);
	
	fwLogPDU(stuff->logFp, RequestPDU);

	if(fwInitRequest(stuff->logFp, Profile, RequestPDU) != 1) {
		netbox_Close(Profile);
		fwFreeInitRequest(RequestPDU);
		sprintf(stuff->Diag,"%s:%s",FWNOSENDINITREQ,Session->Host);
		DisplayError(Session,stuff);
		return(-1);
	}
	fwFreeInitRequest(RequestPDU);

	return(1);
}

/* Totally dependant on session parameters. */
int Search(PSESSION Session,SESSIONSTUFF *stuff, NETBOXPROFILE *Profile) {
	RPNQUERY *RPNQuery;
	QUERY *Z3950Query;
	PDU *RequestPDU;
	
	if(strlen(Session->sQuery)==0) {
		strcpy(stuff->Diag,FWNOQUERYTERM);
		DisplayError(Session,stuff);
		return(-1);
	}

	RPNQuery=fwStrToRPN(Session->sAttributeSet,Session->sQuery);
    	Z3950Query=fwBuildRPNQuery(RPNQuery);
	RequestPDU = fwBuildSearchRequest(Session->sReferenceId,
		Session->sSmallSetUpperBound,
		Session->sLargeSetLowerBound,
		Session->sMedSetPresentNum,
		Session->sReplaceIndicator,
		Session->sResultSetName,
		Session->sDBNames,
		Session->sSmallSetElementSetNames,
		Session->sMediumSetElementSetNames,
		Session->sPreferredRecordSyntax,
		Z3950Query);
	fwLogPDU(stuff->logFp, RequestPDU);
	
	if(fwSearchRequest(stuff->logFp, Profile, RequestPDU) != 1) {
		fwFreeSearchRequest(RequestPDU);
		sprintf(stuff->Diag,"%s:%s,%s",
			FWNOSENDSEARCHREQ,Session->Host,Session->sDBNames);
		DisplayError(Session,stuff);
		return(-1);
	}
	fwFreeSearchRequest(RequestPDU);

	return(1);


}
/* Totally dependant on session parameters. */
int Present(PSESSION Session,SESSIONSTUFF *stuff, NETBOXPROFILE *Profile) {
	PDU *RequestPDU,*ResponsePDU;
	ELEMENTSETNAMES *ESN;
	long recStartPoint=Session->pResultSetStartPoint;
	long numRecsRequested=Session->pNumRecsReq;
	long OrigNumRecsRequested;
	CHAR *tempBuf;
	long len,i;
	int Result;

	/* Error check a bit */
	if(recStartPoint <= 0 || recStartPoint > stuff->numHits)
		return 0;
	if(numRecsRequested <= 0)
		return 0;
	if(numRecsRequested > (stuff->numHits-recStartPoint+1))
		numRecsRequested = stuff->numHits - recStartPoint+1;	

	OrigNumRecsRequested=numRecsRequested;

	/* Encode element set name (only dealing with 1 generic) */
	ESN = (ELEMENTSETNAMES *)calloc(1,sizeof(ELEMENTSETNAMES));
	ESN->which = t_generic;
	ESN->u.generic =
		encodeOctetStr(strdup(Session->pElementSetNames));

	/* Need to loop through present requests until we get 
	   the number of records we want or an error.
	*/
	while(stuff->numRecords < OrigNumRecsRequested) {
		/* Build the PDU */
        	RequestPDU = fwBuildPresentRequest(Session->pReferenceId,
                	Session->pResultSetId, recStartPoint,
			numRecsRequested, ESN,
			Session->pPreferredRecSyntax);

        	fwLogPDU(stuff->logFp, RequestPDU);
	
        	if(fwPresentRequest(stuff->logFp, Profile, RequestPDU) != 1) {
			sprintf(stuff->Diag,"%s:%s, %s",FWNOSENDPRESENTREQ,
				Session->Host, Session->pResultSetId);
			DisplayError(Session,stuff);
                	fwFreePresentRequest(RequestPDU);
                	return(-1);       
        	}                       

		if((ResponsePDU=fwDecodePDU(Profile,MAXNETBUFFERSIZE))==NULL) {
			sprintf(stuff->Diag,"%s:%s, %s",FWNOGETPRESENTRES,
				Session->Host, Session->pResultSetId);
			DisplayError(Session,stuff);
                	fwFreePresentRequest(RequestPDU);
			return(-1);
		}

		fwLogPDU(stuff->logFp,ResponsePDU);

        	if(ResponsePDU->which==t_presentResponse) {
       			StoreRecords(Session, stuff,ResponsePDU);
			if(stuff->Diag[0]!=NULL) {
				DisplayError(Session,stuff);
				return(-1);
			}
		} else {
			/* Unexpected PDU */
			return(-1);
        	}       
		recStartPoint=stuff->numRecords+1;
		numRecsRequested=OrigNumRecsRequested-stuff->numRecords;
	} /* while */
	
	return 1;
}
int GetPrivateProfileStr(CHAR* s1, char* s2, CHAR* s3, CHAR* s4,
		int so, CHAR* s5) {
	return GetPrivateProfileString((char*)s1, (char*)s2, (char*)s3,
			(char*)s4, so, (char*)s5);
}
int ReadProfileSettings(SESSION *S, CHAR *File, CHAR *Group) {
	CHAR temp[MAXSESSIONSTRING];
        if ( CacheProfileFile((char*)File) == 0 ) {
                printf("Opening cache failed.\n");
                return 0;
        }

	GetPrivateProfileStr(Group, "Host",
		S->Host, S->Host,sizeof(S->Host),File);
 
	S->Port = GetPrivateProfileInt(Group, "Port", S->Port, File);
	
	GetPrivateProfileStr(Group, "sQuery",
		S->sQuery, S->sQuery,sizeof(S->sQuery),File);

	GetPrivateProfileStr(Group, "sDBNames",
		S->sDBNames, S->sDBNames,sizeof(S->sDBNames),File);

	S->sSmallSetUpperBound = 
		GetPrivateProfileLong(Group, "sSmallSetUpperBound", 
		S->sSmallSetUpperBound, File);

	S->sLargeSetLowerBound = 
		GetPrivateProfileLong(Group, "sLargeSetLowerBound", 
		S->sLargeSetLowerBound, File);
        
	S->sMedSetPresentNum = 
		GetPrivateProfileLong(Group, "sMedSetPresentNum", 
		S->sMedSetPresentNum, File);

	S->TimeOut = 
		GetPrivateProfileLong(Group, "TimeOut", 
		S->TimeOut, File);

	GetPrivateProfileStr(Group, "OutputFile",
		S->OutputFile, S->OutputFile,sizeof(S->OutputFile),File);

	GetPrivateProfileStr(Group, "StatsFile",
		S->StatsFile, S->StatsFile,sizeof(S->StatsFile),File);

	GetPrivateProfileStr(Group, "LogFile",
		S->LogFile, S->LogFile,sizeof(S->LogFile),File);

	GetPrivateProfileStr(Group, "iAuthentication",
		S->iAuthentication, S->iAuthentication,sizeof(S->iAuthentication),File);

	GetPrivateProfileStr(Group, "iImpID",
		S->iImpID, S->iImpID,sizeof(S->iImpID),File);

	GetPrivateProfileStr(Group, "iImpName",
		S->iImpName, S->iImpName,sizeof(S->iImpName),File);

	GetPrivateProfileStr(Group, "iImpVersion",
		S->iImpVersion, S->iImpVersion,sizeof(S->iImpVersion),File);

	GetPrivateProfileStr(Group, "iProtocolVersion",
		S->iProtocolVersion, S->iProtocolVersion,sizeof(S->iProtocolVersion),File);

	GetPrivateProfileStr(Group, "iOptions",
		S->iOptions, S->iOptions,sizeof(S->iOptions),File);

	S->iPreferredMsgSize = 
		GetPrivateProfileLong(Group, "iPreferredMsgSize", 
		S->iPreferredMsgSize, File);

	S->iMaxRecSize = 
		GetPrivateProfileLong(Group, "iMaxRecSize", 
		S->iMaxRecSize, File);

	GetPrivateProfileStr(Group, "iUserInformationField",
		S->iUserInformationField, S->iUserInformationField,sizeof(S->iUserInformationField),File);

	GetPrivateProfileStr(Group, "iReferenceId",
		S->iReferenceId, S->iReferenceId,sizeof(S->iReferenceId),File);

	GetPrivateProfileStr(Group, "sReferenceId",
		S->sReferenceId, S->sReferenceId,sizeof(S->sReferenceId),File);

	GetPrivateProfileStr(Group, "sResultSetName",
		S->sResultSetName, S->sResultSetName,sizeof(S->sResultSetName),File);

	GetPrivateProfileStr(Group, "sSmallSetElementSetNames",
		S->sSmallSetElementSetNames, S->sSmallSetElementSetNames,sizeof(S->sSmallSetElementSetNames),File);

	GetPrivateProfileStr(Group, "sMediumSetElementSetNames",
		S->sMediumSetElementSetNames, S->sMediumSetElementSetNames,sizeof(S->sMediumSetElementSetNames),File);

	GetPrivateProfileStr(Group, "sPreferredRecordSyntax",
		S->sPreferredRecordSyntax, S->sPreferredRecordSyntax,sizeof(S->sPreferredRecordSyntax),File);

	GetPrivateProfileStr(Group, "sAttributeSet",
		S->sAttributeSet, S->sAttributeSet,sizeof(S->sAttributeSet),File);

	GetPrivateProfileStr(Group, "pReferenceId",
		S->pReferenceId, S->pReferenceId,sizeof(S->pReferenceId),File);

	GetPrivateProfileStr(Group, "pResultSetId",
		S->pResultSetId, S->pResultSetId,sizeof(S->pResultSetId),File);

	S->pResultSetStartPoint = 
		GetPrivateProfileLong(Group, "pResultSetStartPoint", 
		S->pResultSetStartPoint, File);

	S->pNumRecsReq = 
		GetPrivateProfileLong(Group, "pNumRecsReq", 
		S->pNumRecsReq, File);

	GetPrivateProfileStr(Group, "pElementSetNames",
		S->pElementSetNames, S->pElementSetNames,sizeof(S->pElementSetNames),File);

	GetPrivateProfileStr(Group, "pPreferredRecSyntax",
		S->pPreferredRecSyntax, S->pPreferredRecSyntax,sizeof(S->pPreferredRecSyntax),File);

	if ( UncacheProfileFile() == 0 ) {
                printf("Error: changes may not have been saved.\n");
                AbortProfileChanges();
                return;
        }

	return 1;
}

int GetPrivateProfileInt(CHAR *pszSection, char* pszEntry, int pszDefault,
	CHAR *pszFileName) {
	CHAR defaultBuf[MAXSESSIONSTRING];
	CHAR valueBuf[MAXSESSIONSTRING];

	sprintf(defaultBuf,"%i",pszDefault);

	GetPrivateProfileStr(pszSection, pszEntry, defaultBuf,
		valueBuf, sizeof(valueBuf), pszFileName);
	
	return(atoi(valueBuf));
}

long GetPrivateProfileLong(CHAR *pszSection, char *pszEntry, long pszDefault,
	 CHAR *pszFileName) {
	CHAR defaultBuf[MAXSESSIONSTRING];
	CHAR valueBuf[MAXSESSIONSTRING];

	sprintf(defaultBuf,"%ld",pszDefault);

	GetPrivateProfileStr(pszSection, pszEntry, defaultBuf,
		valueBuf, sizeof(valueBuf), pszFileName);
	
	return(atol(valueBuf));
}

int GetCmdLineSettings(SESSION* S, int ac, CHAR** argv) {
/*
	Returns 0 if OK, or a positive index into argv[] that points to the
	error.  Incorrect filenames, groups, and options are not detected
	as errors!  Only syntax errors in command line usage may be
	detected.
*/
	int x;
	CHAR* av;
	CHAR* V;
	CHAR* p;
	CHAR fn[256];
	CHAR group[256];
	CHAR option[256];
	CHAR setting[256];
	int argc = ac - 1;
	if (argc < 1)
		return 0;
	for (x = 1; x <= argc; x++) {
		av = argv[x];
		if ( (av[0] != '-') || ( (av[1] != 'i') && (av[1] != 'o') ) )
			return x;
		V = (CHAR*)malloc(strlen(av+2)+1);
		strcpy(V, av+2);
		switch (av[1]) {
			case 'i':
				if (V[0] == '\0') {
					strcpy(fn, PROFILEFN);
					strcpy(group, PROFILEGROUP);
				} else {
					p = (CHAR*)strchr((char*)V, ',');
					if (p == NULL) {
						strcpy(group, PROFILEGROUP);
					} else {
						*p = '\0';
						*group = '*';
						strcpy(group+1, p+1);
					}
					strcpy(fn, V);
					if (fn[0] == '\0')
						strcpy(fn, PROFILEFN);
				}
				ReadProfileSettings(S, fn, group);
				break;
			case 'o':
				p = (CHAR*)strchr(V, '=');
				if (p == NULL) {
					setting[0] = '\0';
				} else {
					*p = (CHAR)'\0';
					strcpy(setting, p+1);
				}
				strcpy(option, V);
				CacheProfileFile(PROFILEFN);
				WritePrivateProfileString(TEMPGROUP,NULL,NULL, 
						PROFILEFN);
				WritePrivateProfileString(TEMPGROUP, 
					(char*)option, (char*)setting, 
					PROFILEFN);
				UncacheProfileFile();
				ReadProfileSettings(S, (CHAR*)PROFILEFN, 
					(CHAR*)TEMPGROUP);
				WritePrivateProfileString(TEMPGROUP,NULL,NULL, 
						PROFILEFN);
				break;
		} /* switch */
		free(V);
	}
	return 0;
}

void DisplayRecords(SESSION *Session, SESSIONSTUFF *stuff) {
	FILE *fp,*statsFp;
	OCTETSTRING *tempOString;
	CHAR *buf;
	long len,i;
	CHAR errBuf[256];

	if(strlen(Session->StatsFile) > 0) {
		statsFp=fopen(Session->StatsFile,"w");
		if(statsFp!=NULL) {
			/* write num hits and num records */
			fprintf(statsFp,"SUCCESS\n");
			fprintf(statsFp,"%ld\n",stuff->numHits);
			fprintf(statsFp,"%ld\n",stuff->numRecords);
			fclose(statsFp);
		} else {
			perror(errBuf);
			fwLog(stuff->logFp,0,0,
				"Couldn't open stats file: %s",errBuf);
		}
        }
	buf=(CHAR *)calloc(1,Session->iMaxRecSize);
	if(strlen(Session->OutputFile) > 2) 
		fp=fopen(Session->OutputFile,"w");
	else {
		printf("\nThere were %ld records returned!\n",stuff->numRecords);
		printf("You write the record display function here!\n\n");
		printf("Here is an example of how its done...\n\n");
		fp=stdout;
	}

	for(i=0;i<stuff->numRecords;i++) {
		tempOString=stuff->records[i];
		len=tempOString->length;
		if(len>0)
			fwrite(decodeOctetStr(tempOString),1,len,fp);
	}
	if(strlen(Session->OutputFile) > 2) 
		fclose(fp);
	free(buf);
	
}

void DisplayError(SESSION *Session, SESSIONSTUFF *stuff) {
	FILE *statsFp;
	CHAR errBuf[256];

	if(strlen(Session->StatsFile) > 0) {
		statsFp=fopen(Session->StatsFile,"w");
		if(statsFp!=NULL) {
			/* write num hits and num records */
			fprintf(statsFp,"FAILURE\n");
			fprintf(statsFp,"%s\n",stuff->Diag);
			fclose(statsFp);
		} else {
			perror(errBuf);
			fwLog(stuff->logFp,0,0,
				"Couldn't open stats file: %s",errBuf);
		}
        } else {
		printf("%s\n",stuff->Diag);
	}

}
